package de.ozgcloud.alfa.common.attacheditem;

import java.util.stream.Stream;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.itvsh.ozg.pluto.vorgangAttachedItem.GrpcFindVorgangAttachedItemRequest;
import de.itvsh.ozg.pluto.vorgangAttachedItem.GrpcVorgangAttachedItemRequest;
import de.itvsh.ozg.pluto.vorgangAttachedItem.GrpcVorgangAttachedItemResponse;
import de.itvsh.ozg.pluto.vorgangAttachedItem.VorgangAttachedItemServiceGrpc.VorgangAttachedItemServiceBlockingStub;
import de.ozgcloud.alfa.common.GrpcUtil;
import de.ozgcloud.alfa.common.errorhandling.ResourceNotFoundException;
import io.grpc.Status.Code;
import io.grpc.StatusRuntimeException;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Service
class VorgangAttachedItemRemoteService {

	@GrpcClient(GrpcUtil.VORGANG_MANAGER_GRPC_CLIENT)
	private VorgangAttachedItemServiceBlockingStub vorgangAttachedItemServiceStub;

	@Autowired
	private VorgangAttachedItemMapper mapper;

	public Stream<VorgangAttachedItem> findVorgangAttachedItem(String vorgangId, String itemName) {
		var response = vorgangAttachedItemServiceStub.find(buildFindRequest(vorgangId, itemName));

		return response.getVorgangAttachedItemsList().stream().map(mapper::fromGrpc);
	}

	GrpcFindVorgangAttachedItemRequest buildFindRequest(String vorgangId, String itemName) {
		return GrpcFindVorgangAttachedItemRequest.newBuilder()
				.setVorgangId(vorgangId)
				.setItemName(itemName)
				.build();
	}

	public VorgangAttachedItem getById(String id) {
		var response = doGetById(id);

		return mapper.fromGrpc(response.getVorgangAttachedItem());
	}

	GrpcVorgangAttachedItemResponse doGetById(String id) {
		try {
			return vorgangAttachedItemServiceStub.getById(buildGetByIdRequest(id));
		} catch (StatusRuntimeException e) {
			if (e.getStatus().getCode() == Code.NOT_FOUND) {
				throw new ResourceNotFoundException(VorgangAttachedItem.class, id);
			}
			throw e;
		}
	}

	GrpcVorgangAttachedItemRequest buildGetByIdRequest(String id) {
		return GrpcVorgangAttachedItemRequest.newBuilder().setId(id).build();
	}
}