/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.user;

import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.stereotype.Service;

import de.itvsh.kop.user.grpc.userprofile.GrpcGetUserProfileRequest;
import de.itvsh.kop.user.grpc.userprofile.UserProfileServiceGrpc;
import de.itvsh.kop.user.grpc.userprofile.UserProfileServiceGrpc.UserProfileServiceBlockingStub;
import io.grpc.Channel;
import io.grpc.Status;
import io.grpc.StatusRuntimeException;
import lombok.extern.log4j.Log4j2;

@ConditionalOnProperty(name = "grpc.client.user-manager.address")
@Log4j2
@Service
class UserRemoteService {

	@Autowired
	private Channel userManagerChannel;

	@Autowired
	private UserProfileMapper mapper;

	public UserProfile getById(UserId id) {
		var response = getUserProfileServiceStub().getById(buildRequest(id));
		return mapper.mapFrom(response.getUserProfile());
	}

	public Optional<UserProfile> getByExternalId(UserId externalUserId) {
		try {
			var response = getUserProfileServiceStub().getByExternalId(buildRequest(externalUserId));
			return Optional.of(mapper.mapFrom(response.getUserProfile()));
		} catch (StatusRuntimeException e) {
			if (isNotFound(e)) {
				return Optional.empty();
			}
			LOG.error("Error loading internal Userid.", e);
			throw new UserManagerUnavailableException();
		}
	}

	private GrpcGetUserProfileRequest buildRequest(UserId externalUserId) {
		return GrpcGetUserProfileRequest.newBuilder().setUserId(externalUserId.toString()).build();
	}

	private boolean isNotFound(StatusRuntimeException e) {
		return e.getStatus().getCode().value() == Status.NOT_FOUND.getCode().value();
	}

	UserProfileServiceBlockingStub getUserProfileServiceStub() {
		return UserProfileServiceGrpc.newBlockingStub(userManagerChannel);
	}
}