/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.kommentar;

import java.time.ZonedDateTime;
import java.util.Map;

import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.MappingConstants;
import org.mapstruct.NullValueCheckStrategy;
import org.mapstruct.ReportingPolicy;
import org.mapstruct.ValueMapping;
import org.springframework.beans.factory.annotation.Autowired;

import de.itvsh.kop.pluto.common.grpc.GrpcObjectMapper;
import de.itvsh.ozg.pluto.grpc.command.GrpcCommand;
import de.itvsh.ozg.pluto.vorgangAttachedItem.GrpcVorgangAttachedItem;
import de.ozgcloud.alfa.attachment.AttachmentMapper;

@Mapper(unmappedTargetPolicy = ReportingPolicy.ERROR, nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
abstract class KommentarMapper {

	static final String ID = "id";
	static final String TEXT = "text";
	static final String CREATED_BY = "createdBy";
	static final String CREATED_AT = "createdAt";
	static final String ATTACHMENTS = "attachments";

	@Autowired
	private GrpcObjectMapper grpcObjectMapper;

	@Mapping(target = "kommentar", ignore = true)
	@ValueMapping(source = "UNRECOGNIZED", target = MappingConstants.NULL)
	@ValueMapping(source = "UNDEFINED", target = MappingConstants.NULL)
	abstract KommentarCommand toKommentarCommand(GrpcCommand command);

	public Kommentar fromItem(GrpcVorgangAttachedItem vorgangAttachedItem) {
		return fromItemMap(grpcObjectMapper.mapFromGrpc(vorgangAttachedItem.getItem()), vorgangAttachedItem.getVersion());
	}

	Kommentar fromItemMap(Map<String, Object> map, long version) {
		return Kommentar.builder()
				.id((String) map.get(ID))
				.version(version)
				.createdAt(ZonedDateTime.parse((String) map.get(CREATED_AT)))
				.createdBy((String) map.get(CREATED_BY))
				.text((String) map.get(TEXT))
				.attachments(AttachmentMapper.mapAttachments(map.get(ATTACHMENTS)))
				.build();
	}
}
