/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.kommentar;

import java.util.stream.Stream;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.itvsh.ozg.pluto.vorgangAttachedItem.GrpcFindVorgangAttachedItemRequest;
import de.itvsh.ozg.pluto.vorgangAttachedItem.GrpcVorgangAttachedItemRequest;
import de.itvsh.ozg.pluto.vorgangAttachedItem.VorgangAttachedItemServiceGrpc.VorgangAttachedItemServiceBlockingStub;
import de.ozgcloud.alfa.common.GrpcUtil;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Service
class KommentarRemoteService {

	static final String ITEM_NAME = "Kommentar";

	@GrpcClient(GrpcUtil.VORGANG_MANAGER_GRPC_CLIENT)
	private VorgangAttachedItemServiceBlockingStub vorgangAttachedItemServiceStub;
	@Autowired
	private KommentarMapper mapper;

	public Stream<Kommentar> findByVorgangId(String vorgangId) {
		var response = vorgangAttachedItemServiceStub.find(buildFindRequest(vorgangId));

		return response.getVorgangAttachedItemsList().stream().map(mapper::fromItem);
	}

	GrpcFindVorgangAttachedItemRequest buildFindRequest(String vorgangId) {
		return GrpcFindVorgangAttachedItemRequest.newBuilder()
				.setVorgangId(vorgangId)
				.setItemName(ITEM_NAME)
				.build();
	}

	public Kommentar getById(String kommentarId) {
		var response = vorgangAttachedItemServiceStub.getById(buildRequest(kommentarId));

		return mapper.fromItem(response.getVorgangAttachedItem()).toBuilder().vorgangId(response.getVorgangAttachedItem().getVorgangId()).build();
	}

	private GrpcVorgangAttachedItemRequest buildRequest(String kommentarId) {
		return GrpcVorgangAttachedItemRequest.newBuilder().setId(kommentarId).build();
	}
}
