package de.ozgcloud.alfa.loeschanforderung;

import java.util.Map;

import org.apache.commons.collections.MapUtils;
import org.springframework.stereotype.Component;

import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItem;
import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandBodyMapper;
import de.ozgcloud.alfa.common.user.UserId;
import de.ozgcloud.alfa.vorgang.Vorgang.VorgangStatus;

@Component
class LoeschAnforderungMapper {

	public LoeschAnforderung fromVorgangAttachedItem(VorgangAttachedItem vorgangAttachedItem) {
		var item = vorgangAttachedItem.getItem();
		return LoeschAnforderung.builder()
				.id(vorgangAttachedItem.getId())
				.vorgangId(vorgangAttachedItem.getVorgangId())
				.requestedBy(getRequestedBy(item))
				.prevStatus(getPrevState(item))
				.build();
	}

	public LoeschAnforderung fromCommand(Command command) {
		var item = getItem(command);
		return LoeschAnforderung.builder()
				.id(command.getRelationId())
				.vorgangId(command.getVorgangId())
				.requestedBy(getRequestedBy(item))
				.prevStatus(getPrevState(item))
				.build();
	}

	@SuppressWarnings("unchecked")
	private Map<String, Object> getItem(Command command) {
		return MapUtils.getMap(command.getBody(), CommandBodyMapper.ITEM_PROPERTY);
	}

	private UserId getRequestedBy(Map<String, Object> item) {
		return UserId.from(MapUtils.getString(item, LoeschAnforderung.REQUESTED_BY_FIELD));
	}

	private VorgangStatus getPrevState(Map<String, Object> item) {
		return VorgangStatus.valueOf(MapUtils.getString(item, LoeschAnforderung.PREV_STATUS_FIELD));
	}
}
