package de.ozgcloud.alfa.statistic;

import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.itvsh.ozg.pluto.statistic.GrpcVorgangCountRequest;
import de.itvsh.ozg.pluto.statistic.GrpcVorgangStatisticQuery;
import de.itvsh.ozg.pluto.statistic.GrpcVorgangStatisticQuery.GroupMethod;
import de.itvsh.ozg.pluto.statistic.GrpcVorgangStatisticQuery.Operator;
import de.itvsh.ozg.pluto.statistic.GrpcVorgangStatisticRequest;
import de.itvsh.ozg.pluto.statistic.GrpcVorgangStatisticResponse;
import de.itvsh.ozg.pluto.statistic.StatisticServiceGrpc.StatisticServiceBlockingStub;
import de.ozgcloud.alfa.common.GrpcUtil;
import de.ozgcloud.alfa.vorgang.Vorgang.VorgangStatus;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Service
class StatisticRemoteService {

	static final String COUNT_VORGANG_STATUS_RESULT_NAME_FORMAT = "countVorgangStatus_%s";
	static final String COUNT_WIEDERVORLAGE_NEXT_FRIST_RESULT_NAME = "countWiedervorlage";
	static final String EXISTS_WIEDERVORLAGE_OVERDUE_RESULT_NAME = "existsWiedervorlageOverdue";

	static final String STATUS_PATH = "Vorgang.status";
	static final String WIEDERVORLAGE_NEXT_FRIST_PATH = "ClientAttribute.Goofy.nextWiedervorlageFrist";

	static final String OPERAND_TODAY_DATE = "today()";

	@GrpcClient(GrpcUtil.VORGANG_MANAGER_GRPC_CLIENT)
	private StatisticServiceBlockingStub statisticServiceStub;
	@Autowired
	private StatisticMapper mapper;

	public Map<String, StatisticResult> getVorgaengeStatistics(List<VorgangStatus> countByVorgangStatus) {
		var grpcRequest = GrpcVorgangStatisticRequest.newBuilder()
				.addQuery(buildExistsWiedervorlageOverdueQuery())
				.addAllQuery(buildCountByStatusQueries(countByVorgangStatus))
				.build();

		var grpcResponse = statisticServiceStub.getVorgangStatistic(grpcRequest);

		return buildStatisticResult(grpcResponse);
	}

	List<GrpcVorgangStatisticQuery> buildCountByStatusQueries(List<VorgangStatus> countByVorgangStatus) {
		return countByVorgangStatus.stream().map(this::buildCountByVorgangStatusQuery).toList();
	}

	GrpcVorgangStatisticQuery buildCountByVorgangStatusQuery(VorgangStatus vorgangStatus) {
		return GrpcVorgangStatisticQuery.newBuilder()
				.setResultName(buildCountByStatusResultName(vorgangStatus))
				.setPath(STATUS_PATH)
				.setGroupMethod(GroupMethod.COUNT)
				.setOperator(Operator.EQUAL)
				.setOperandStringValue(vorgangStatus.name())
				.build();
	}

	GrpcVorgangStatisticQuery buildExistsWiedervorlageOverdueQuery() {
		return GrpcVorgangStatisticQuery.newBuilder()
				.setResultName(EXISTS_WIEDERVORLAGE_OVERDUE_RESULT_NAME)
				.setPath(WIEDERVORLAGE_NEXT_FRIST_PATH)
				.setGroupMethod(GroupMethod.EXISTS)
				.setOperator(Operator.LESS_THEN_OR_EQUAL_TO)
				.setOperandStringValue(OPERAND_TODAY_DATE)
				.build();
	}

	public String buildCountByStatusResultName(VorgangStatus vorgangStatus) {
		return String.format(COUNT_VORGANG_STATUS_RESULT_NAME_FORMAT, vorgangStatus.name());
	}

	Map<String, StatisticResult> buildStatisticResult(GrpcVorgangStatisticResponse grpcVorgangStatisticResponse) {
		return grpcVorgangStatisticResponse.getResultList()
				.stream()
				.map(mapper::toResult)
				.collect(Collectors.toMap(StatisticResult::getName, Function.identity()));
	}

	// TODO sebo: der Code muss später gelöscht werden
	public CountVorgangResponse countVorgang(List<String> countByPath) {
		var response = statisticServiceStub.countVorgang(buildVorgangCountRequest(countByPath));

		return mapper.fromCountVorgangResponse(response);
	}

	GrpcVorgangCountRequest buildVorgangCountRequest(List<String> countByPath) {
		return GrpcVorgangCountRequest.newBuilder().addAllCountByPath(countByPath).build();
	}
	// END: code löschen
}