/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.binaryfile;

import static org.mockito.Mockito.*;

import java.io.IOException;
import java.io.OutputStream;
import java.util.List;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.alfa.common.file.OzgFileTestFactory;

class BinaryFileServiceTest {

	@InjectMocks
	private BinaryFileService service;

	@Mock
	private BinaryFileRemoteService remoteService;

	@Nested
	class TestUploadFile {

		@Test
		void shouldCallRemoteService() throws IOException {
			var uploadBinaryFile = UploadBinaryFileTestFactory.create();

			service.uploadFile(uploadBinaryFile);

			verify(remoteService).uploadFile(uploadBinaryFile);
		}
	}

	@Nested
	class TestWriteFileContent {

		@Mock
		private OutputStream out;

		@Test
		void shouldCallRemotService() {
			service.writeFileContent(OzgFileTestFactory.ID, out);

			verify(remoteService).writeFileContent(OzgFileTestFactory.ID, out);
		}
	}

	@Nested
	class TestGetFile {

		@Test
		void shouldCallRemoteService() {
			service.getFile(BinaryFileTestFactory.FILE_ID);

			verify(remoteService).getFile(BinaryFileTestFactory.FILE_ID);
		}
	}

	@Nested
	class TestGetFiles {

		@Test
		void shouldCallRemotService() {
			var fileIds = List.of(BinaryFileTestFactory.FILE_ID);

			service.getFiles(fileIds);

			verify(remoteService).getFiles(fileIds);
		}
	}
}