/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.kommentar;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.time.ZonedDateTime;
import java.util.UUID;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentMatchers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import de.ozgcloud.alfa.kommentar.KommentarController.KommentarByVorgangController;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;

class KommentarByVorgangControllerTest {

	@InjectMocks
	private KommentarByVorgangController controller;

	@Mock
	private KommentarService service;

	@Mock
	private KommentarModelAssembler modelAssembler;

	private MockMvc mockMvc;

	@BeforeEach
	void initMockMvc() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).build();
	}

	@Nested
	class TestGetAll {

		@Test
		void shouldCallEndpoint() throws Exception {
			ResultActions callEndpoint = callEndpoint();

			callEndpoint.andExpect(status().isOk());
		}

		@Test
		void shouldCallService() throws Exception {
			callEndpoint();

			verify(service).findByVorgangId(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldCallModelAssembler() throws Exception {
			callEndpoint();

			verify(modelAssembler).toCollectionModel(ArgumentMatchers.<Stream<Kommentar>>any(), anyString());
		}

		private ResultActions callEndpoint() throws Exception {
			return mockMvc.perform(get(KommentarByVorgangController.KOMMENTAR_BY_VORGANG_PATH + "/" + VorgangHeaderTestFactory.ID + "/kommentars"));
		}

		@Test
		void shouldSortByCreatedAt() {
			var firstId = UUID.randomUUID().toString();
			Kommentar first = KommentarTestFactory.createBuilder().id(firstId).createdAt(ZonedDateTime.now()).build();

			var kommentarList = controller.sortByCreatedAt(Stream.of(KommentarTestFactory.create(), first)).collect(Collectors.toList());

			assertThat(kommentarList.get(0).getId()).isEqualTo(first.getId());
			assertThat(kommentarList.get(1).getId()).isEqualTo(KommentarTestFactory.ID);
		}
	}
}