package de.ozgcloud.alfa.postfach;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import de.ozgcloud.alfa.vorgang.VorgangWithEingangTestFactory;

class PostfachNachrichtHelperTest {

	@InjectMocks
	private PostfachNachrichtHelper helper;
	@Mock
	private PostfachMailMapper mapper;

	@DisplayName("Add postfachAddress to body")
	@Nested
	class TestAddPostfachAddressToBody {

		private final PostfachMail postfachNachrichtWithoutPostfachAddress = PostfachMailTestFactory.createBuilder().postfachAddress(null).build();
		private final VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();

		@BeforeEach
		void mockMapper() {
			when(mapper.mapPostfachAddressToMap(any())).thenReturn(PostfachAddressTestFactory.createAsMap());
		}

		@Test
		void shouldCallMapperWithPostfachAddress() {
			helper.addPostfachAddressToBody(postfachNachrichtWithoutPostfachAddress, vorgang);

			verify(mapper).mapPostfachAddressToMap(vorgang.getHeader().getServiceKonto().getPostfachAddresses().get(0));
		}

		@Test
		void shouldReturn() {
			var body = helper.addPostfachAddressToBody(postfachNachrichtWithoutPostfachAddress, vorgang);

			assertThat(((PostfachMail) body).getPostfachAddress()).usingRecursiveComparison().isEqualTo(PostfachAddressTestFactory.createAsMap());
		}
	}
}
