/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.vorgang;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;
import org.mockito.Spy;

import de.itvsh.kop.pluto.common.grpc.GrpcFormDataMapper;
import de.ozgcloud.alfa.common.StringToNullMapper;

class AntragstellerMapperTest {

	@InjectMocks
	private final AntragstellerMapper mapper = Mappers.getMapper(AntragstellerMapper.class);
	@Spy
	private final GrpcFormDataMapper grpcFormDataMapper = Mappers.getMapper(GrpcFormDataMapper.class);
	@Spy
	private final StringToNullMapper stringToNullMapper = Mappers.getMapper(StringToNullMapper.class);

	@Test
	void shouldMap() {
		var antragsteller = mapper.toAntragsteller(GrpcAntragstellerTestFactory.create());

		assertThat(antragsteller).usingRecursiveComparison().ignoringFields("otherData").isEqualTo(AntragstellerTestFactory.create());
	}

	@Test
	void shouldCallGrpcFormDataMapper() {
		mapper.toAntragsteller(GrpcAntragstellerTestFactory.create());

		verify(grpcFormDataMapper).mapFromFormData(GrpcAntragstellerTestFactory.OTHER_DATA);
	}

	@Test
	void shouldCallStringToNullMapper() {
		mapper.toAntragsteller(GrpcAntragstellerTestFactory.create());

		verify(stringToNullMapper, atLeastOnce()).fromString(any());
	}
}