package de.ozgcloud.alfa.export;

import static java.util.Optional.*;

import java.time.format.DateTimeFormatter;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import de.xoev.xdomea.FeldType;
import de.xoev.xdomea.FeldgruppeType;

@Component
class FeldGruppeTypeCreator {

	static final String NAME = "FeldgruppeOZGCloudBasis";
	static final String BESCHREIBUNG = "Feldgruppe für OZGCloud Basis";

	public FeldgruppeType create(VorgangWithEingang vorgang) {
		var feldgruppe = new FeldgruppeType();
		feldgruppe.setName(NAME);
		feldgruppe.setBeschreibung(BESCHREIBUNG);
		feldgruppe.getFeld().addAll(createFeldType(vorgang));
		return feldgruppe;
	}

	List<FeldType> createFeldType(VorgangWithEingang vorgang) {
		var antragsteller = ofNullable(vorgang.getEingang()).flatMap(eingang -> ofNullable(eingang.getAntragsteller()));
		return List.of(
				ExportFelder.LEIKA_ID.createFeld(),
				ExportFelder.DATUM_ANTRAGSEINGANG.createFeld(DateTimeFormatter.ISO_DATE_TIME.format(vorgang.getCreatedAt())),
				ExportFelder.NAME.createFeld(antragsteller.flatMap(a -> ofNullable(a.getNachname())).orElse(StringUtils.EMPTY)),
				ExportFelder.VORNAME.createFeld(antragsteller.flatMap(a -> ofNullable(a.getVorname())).orElse(StringUtils.EMPTY)),
				ExportFelder.GEBURTSDATUM.createFeld(antragsteller.flatMap(a -> ofNullable(a.getGeburtsdatum())).orElse(StringUtils.EMPTY)),
				ExportFelder.PLZ.createFeld(antragsteller.flatMap(a -> ofNullable(a.getPlz())).orElse(StringUtils.EMPTY)));
	}

}
