package de.ozgcloud.alfa.export;

import static org.assertj.core.api.Assertions.*;

import java.util.Optional;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;
import org.mockito.InjectMocks;
import org.mockito.MockedStatic;
import org.mockito.Mockito;

import de.itvsh.kop.common.errorhandling.TechnicalException;

class DateConverterTest {

	@InjectMocks
	private DateConverter converter;

	@Nested
	class TestConvertGermanFormatToISO {

		private static final int YEAR = 2023;
		private static final int MONTH = 2;
		private static final int DAY = 14;

		private static final String DATE = String.format("%s.%s%s.%s", DAY, 0, MONTH, YEAR);

		@Test
		void shouldConvert() {
			var converted = callConverter(DATE);

			assertThat(converted).get().extracting(XMLGregorianCalendar::getYear, XMLGregorianCalendar::getMonth, XMLGregorianCalendar::getDay)
					.containsExactly(YEAR, MONTH, DAY);
		}

		@ParameterizedTest
		@ValueSource(strings = { "2005-10-10", "abc", "19. February 1970", " ", "" })
		void shouldReturnEmpty(String dateStr) {
			var converted = callConverter(dateStr);

			assertThat(converted).isEmpty();
		}

		@Test
		void shouldReturnEmptyIfDateIsNull() {
			var converted = callConverter(null);

			assertThat(converted).isEmpty();
		}

		@Test
		void shouldThrowTechnicalException() {
			try (MockedStatic<DatatypeFactory> datatypeFactory = Mockito.mockStatic(DatatypeFactory.class)) {
				datatypeFactory.when(DatatypeFactory::newInstance).thenThrow(DatatypeConfigurationException.class);

				assertThatThrownBy(() -> callConverter(DATE)).isInstanceOf(TechnicalException.class);
			}
		}

		private Optional<XMLGregorianCalendar> callConverter(String dateStr) {
			return converter.convertGermanFormatToISO(dateStr);
		}
	}

}