package de.ozgcloud.alfa.export;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.io.OutputStream;
import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.springframework.http.MediaType;

import de.ozgcloud.alfa.common.binaryfile.BinaryFileService;
import de.ozgcloud.alfa.common.binaryfile.FileId;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.common.file.OzgFileService;
import de.ozgcloud.alfa.common.file.OzgFileTestFactory;
import de.ozgcloud.alfa.vorgang.EingangTestFactory;

class ExportFileServiceTest {

	@InjectMocks
	private ExportFileService service;

	@Mock
	private OzgFileService ozgFileService;

	@Mock
	private BinaryFileService binaryFileService;

	@Nested
	class TestGetAllPdfs {

		private final OzgFile representationPdfFile = OzgFileTestFactory.createBuilder().contentType(MediaType.APPLICATION_PDF_VALUE).build();
		private final OzgFile representationXmlFile = OzgFileTestFactory.createBuilder().contentType(MediaType.TEXT_XML_VALUE).build();
		private final OzgFile attachmentPdfFile = OzgFileTestFactory.createBuilder().contentType(MediaType.APPLICATION_PDF_VALUE).build();
		private final OzgFile attachmentXmlFile = OzgFileTestFactory.createBuilder().contentType(MediaType.TEXT_XML_VALUE).build();

		@BeforeEach
		void init() {
			when(ozgFileService.getRepresentationsByEingang(EingangTestFactory.ID)).thenReturn(
					Stream.of(representationPdfFile, representationXmlFile));
			when(ozgFileService.getAttachmentsByEingang(EingangTestFactory.ID)).thenReturn(Stream.of(attachmentXmlFile, attachmentPdfFile));
		}

		@Test
		void shouldLoadRepresentations() {
			service.getAllPdfs(EingangTestFactory.ID);

			verify(ozgFileService).getRepresentationsByEingang(EingangTestFactory.ID);
		}

		@Test
		void shouldLoadAttachments() {
			service.getAllPdfs(EingangTestFactory.ID);

			verify(ozgFileService).getAttachmentsByEingang(EingangTestFactory.ID);
		}

		@Test
		void shouldReturnPdfs() {
			var ozgFiles = service.getAllPdfs(EingangTestFactory.ID);

			assertThat(ozgFiles).hasSize(2).containsExactlyInAnyOrder(attachmentPdfFile, representationPdfFile);
		}
	}

	@Nested
	class TestWriteOzgFile {

		private final FileId fileId = OzgFileTestFactory.ID;

		@Mock
		private OutputStream outputStream;

		@Test
		void shouldCallBinaryFileService() {
			service.writeOzgFile(fileId, outputStream);

			verify(binaryFileService).writeFileContent(fileId, outputStream);
		}
	}

}