package de.ozgcloud.alfa.export;

import static de.ozgcloud.alfa.common.UserProfileUrlProviderTestFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.Link;

import de.ozgcloud.alfa.common.FeatureToggleProperties;
import de.ozgcloud.alfa.common.UserProfileUrlProvider;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangWithEingangTestFactory;

class ExportVorgangProzessorTest {

	@InjectMocks
	private ExportVorgangProzessor prozessor;

	@Mock
	private FeatureToggleProperties featureToggleProperties;

	private UserProfileUrlProvider urlProvider = new UserProfileUrlProvider();

	@Nested
	class TestProcess {

		@DisplayName("Export link")
		@Nested
		class TestExportLink {

			@BeforeEach
			void init() {
				initUserProfileUrlProvider(urlProvider);
			}

			@Test
			void shouldAddLink() {
				when(featureToggleProperties.isVorgangExport()).thenReturn(true);

				var model = prozessor.process(EntityModel.of(VorgangWithEingangTestFactory.create()));

				assertThat(model.getLink(ExportVorgangProzessor.REL_EXPORT)).isPresent().get().extracting(Link::getHref)
						.isEqualTo(ExportVorgangController.PATH + "/" + VorgangHeaderTestFactory.ID);
			}

			@Test
			void shouldNotAddLink() {
				when(featureToggleProperties.isVorgangExport()).thenReturn(false);

				var model = prozessor.process(EntityModel.of(VorgangWithEingangTestFactory.create()));

				assertThat(model.getLink(ExportVorgangProzessor.REL_EXPORT)).isEmpty();
			}
		}

	}

}