package de.ozgcloud.alfa.export;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Map;

import jakarta.xml.bind.Marshaller;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.oxm.jaxb.Jaxb2Marshaller;

class XmlMarshallerConfigurationTest {

	@Spy
	@InjectMocks
	private XDomeaXmlMarshallerConfiguration configuration;

	@Mock
	private XDomeaNamespacePrefixMapper prefixMapper;

	@Nested
	class TestCreateMarshallerProperties {

		@Test
		void shouldFromatMarshalledXml() {
			var props = configuration.createMarshallerProperties();

			assertThat(props).containsEntry(Marshaller.JAXB_FORMATTED_OUTPUT, XDomeaXmlMarshallerConfiguration.JAXB_FORMATTED_OUTPUT);
		}

		@Test
		void shouldUseNamespacePrefixMapper() {
			var props = configuration.createMarshallerProperties();

			assertThat(props).containsEntry(XDomeaXmlMarshallerConfiguration.PROPERTY_NAMESPACE_PREFIX_MAPPER, prefixMapper);
		}
	}

	@Nested
	class TestMarshaller {

		private final Map<String, Object> marshallerProps = Map.of();

		@BeforeEach
		void init() {
			doReturn(marshallerProps).when(configuration).createMarshallerProperties();
		}

		@Test
		void shouldSetContextPath() {
			var marshaller = configuration.marshaller();

			assertThat(marshaller.getContextPath()).isEqualTo(XDomeaXmlMarshallerConfiguration.CONTEXT_PATH);
		}

		@Test
		void shouldCreateMarshallerProperties() {
			configuration.marshaller();

			verify(configuration).createMarshallerProperties();
		}

		@Test
		void shouldReturnMarshaller() {
			var marshaller = configuration.marshaller();

			assertThat(marshaller).isNotNull().isInstanceOf(Jaxb2Marshaller.class);
		}
	}

}