/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { VorgangForwardingE2EComponent } from '../../../components/vorgang/vorgang-forward.e2e.component';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { VorgangE2E, VorgangStatusE2E, vorgangStatusLabelE2E } from '../../../model/vorgang';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { dropCollections } from '../../../support/cypress-helper';
import { exist, haveText, haveValue, notExist } from '../../../support/cypress.util';
import { FORWARDING_INVALID_EMAIL, FORWARDING_TEST_EMAIL, VORGANG_ZUSTAENDIGE_STELLE_EMAIL } from '../../../support/data.util';
import { loginAsEmil } from '../../../support/user-util';
import { buildVorgang, createVorgang, initVorgaenge, objectIds } from '../../../support/vorgang-util';

describe.skip('Vorgang forward', () => {
	const mainPage: MainPage = new MainPage();
	const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

	const vorgangPage: VorgangPage = new VorgangPage();
	const forwardingContainer: VorgangForwardingE2EComponent = vorgangPage.getForwardingContainer();

	const vorgang: VorgangE2E = { ...createVorgang(), name: 'TestVorgangRedirectSpinner' };
	const vorgangEmpty: VorgangE2E = buildVorgang(objectIds[0], 'TestVorgangRedirectEmpty');

	before(() => {
		initVorgaenge([vorgang, vorgangEmpty]);

		loginAsEmil();

		waitForSpinnerToDisappear();
		exist(vorgangList.getRoot());
	})

	after(() => {
		dropCollections();
	})

	describe('after click on vorgang', () => {

		it('should show vorgang header', () => {
			vorgangList.getListItem(vorgang.name).getRoot().click();
			waitForSpinnerToDisappear();

			exist(vorgangPage.getVorgangDetailHeader().getRoot());
		})

		it('should show forwarding', () => {
			exist(forwardingContainer.getRoot());
		})

		it('should be prefilled', () => {
			haveValue(forwardingContainer.getFormular().getZustaendigeStelle(), VORGANG_ZUSTAENDIGE_STELLE_EMAIL);
		})
	})

	describe('fill forward formular', () => {

		describe('with email domain NOT in landesnetz', () => {

			it('should show validation message for invalid password', () => {
				const forwardFormular = forwardingContainer.getFormular();
				forwardFormular.getZustaendigeStelle().clear().type(FORWARDING_INVALID_EMAIL);//TODO create valid email-address without adding it to landesnetzlist
				forwardFormular.getPasswort().clear().type('zukurz');

				forwardFormular.forward();
				waitForSpinnerToDisappear();

				exist(forwardFormular.getPasswortError());
			})

			it('should show validation message for invalid email', () => {
				const forwardFormular = forwardingContainer.getFormular();
				forwardFormular.getZustaendigeStelle().clear().type(FORWARDING_INVALID_EMAIL);

				forwardFormular.forward();
				waitForSpinnerToDisappear();

				exist(forwardFormular.getZustaendigeStelleError());
			})
		})

		describe('with email domain in landesnetz', () => {

			it('should send formular with landesnetz email and empty password', () => {
				const forwardFormular = forwardingContainer.getFormular();
				forwardFormular.getZustaendigeStelle().clear().type(FORWARDING_TEST_EMAIL);
				forwardFormular.getPasswort().clear();

				forwardFormular.forward();

				exist(forwardingContainer.getSpinner());
			})
		})
	})

	describe('success forwarding', () => {

		it('should have status ' + vorgangStatusLabelE2E[VorgangStatusE2E.WEITERGELEITET], { defaultCommandTimeout: 30000 }, () => {
			waitForSpinnerToDisappear();

			haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.WEITERGELEITET])
		})

		it('should show forward success text', () => {
			exist(forwardingContainer.getForwardingSuccessMessage());
		})

		it('should show "mark as success" button', () => {
			exist(forwardingContainer.getMarkAsSuccessButton());
		})

		it('should NOT show formular', () => {
			notExist(forwardingContainer.getFormular().getRoot());
		})

		it('should show "zustellung success message" after click on button', () => {
			forwardingContainer.getMarkAsSuccessButton().click();
			waitForSpinnerToDisappear();

			exist(forwardingContainer.getForwardingSuccessMessage());
		})
	})

	describe('switch to list view', () => {

		it('should have status ' + vorgangStatusLabelE2E[VorgangStatusE2E.WEITERGELEITET] + ' in list', () => {
			vorgangPage.getSubnavigation().getBackButton().click();
			waitForSpinnerToDisappear();

			haveText(vorgangList.getListItem(vorgang.name).getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.WEITERGELEITET]);
		})
	})

	describe('change to non forwarding vorgang', () => {

		it('should formular after switching vorgang without pending commands', () => {
			vorgangList.getListItem(vorgangEmpty.name).getRoot().click();
			waitForSpinnerToDisappear();

			exist(forwardingContainer.getRoot());
		})

		it('should show forward formular', () => {
			exist(forwardingContainer.getFormular().getRoot());
		})

		it('sould not show forwarding success text', () => {
			notExist(forwardingContainer.getForwardingSuccessMessage());
		})
	})
})
