/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { SnackBarE2EComponent } from '../../../components/ui/snackbar.e2e.component';
import { VorgangDetailHeaderE2EComponent } from '../../../components/vorgang/vorgang-detail-header.e2e.component';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { VorgangSubnavigationE2EComponent } from '../../../components/vorgang/vorgang-subnavigation';
import { HttpMethodE2E } from '../../../model/util';
import { VorgangE2E } from '../../../model/vorgang';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { dropCollections, interceptWithResponse, waitOfInterceptor } from '../../../support/cypress-helper';
import { contains, exist, notExist } from '../../../support/cypress.util';
import { MessagesE2E } from '../../../support/messages';
import { loginAsSabine } from '../../../support/user-util';
import { createVorgang, initVorgang } from '../../../support/vorgang-util';

describe('Interceptor Forbidden', () => {
	const mainPage: MainPage = new MainPage();
	const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();
	const snackbar: SnackBarE2EComponent = mainPage.getSnackBar();

	const vorgangPage: VorgangPage = new VorgangPage();
	const vorgangHeader: VorgangDetailHeaderE2EComponent = vorgangPage.getVorgangDetailHeader();
	const subnavigation: VorgangSubnavigationE2EComponent = vorgangPage.getSubnavigation();

	const vorgang: VorgangE2E = createVorgang();

	before(() => {
		initVorgang(vorgang);

		loginAsSabine();

		waitForSpinnerToDisappear();
		exist(vorgangList.getRoot());
	})

	after(() => {
		dropCollections();
	})

	describe('click on vorgang', () => {

		const statusCode: number = 403;
		const interceptor: string = 'getVorgang';

		beforeEach(() => {
			const url: string = '*/vorgangs/' + vorgang._id.$oid;

			interceptWithResponse(HttpMethodE2E.GET, url, { statusCode, body: { issues: [{ message: '1', messageCode: '2' }] } }).as(interceptor);
		})

		it('should show page and snackbar', () => {
			vorgangList.getListItem(vorgang.name).getRoot().click();
			waitForSpinnerToDisappear();

			waitOfInterceptor(interceptor).then((interception) => {
				assert(interception.response.statusCode, statusCode.toString());
				notExist(vorgangHeader.getRoot());
				contains(snackbar.getMessage(), MessagesE2E.HTTP_STATUS_FORBIDDEN);
				exist(snackbar.getCloseButton());
				notExist(snackbar.getRevokeButton());
				notExist(vorgangPage.getSpinner());
			})
		})

		it('should close snackbar on close button', () => {
			snackbar.getCloseButton().click()

			notExist(snackbar.getMessage());
		})

		it('should navigate back on back button', () => {
			subnavigation.getBackButton().click();

			waitForSpinnerToDisappear();
			exist(vorgangList.getRoot());
		})
	})
})