/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { VorgangFormularButtonsE2EComponent } from 'apps/goofy-e2e/src/components/vorgang/vorgang-formular-buttons.e2e.components';
import { SnackBarE2EComponent } from '../../../components/ui/snackbar.e2e.component';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { VorgangE2E, VorgangMessagesE2E, VorgangStatusE2E, vorgangStatusLabelE2E } from '../../../model/vorgang';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { dropCollections } from '../../../support/cypress-helper';
import { contains, exist, haveText, notExist } from '../../../support/cypress.util';
import { loginAsSabine } from '../../../support/user-util';
import { buildVorgang, createVorgang, initVorgaenge, objectIds } from '../../../support/vorgang-util';

describe('Vorgang bearbeiten', () => {
	const mainPage: MainPage = new MainPage();
	const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();
	const snackBar: SnackBarE2EComponent = mainPage.getSnackBar();

	const vorgangPage: VorgangPage = new VorgangPage();
	const vorgangFormularButtons: VorgangFormularButtonsE2EComponent = vorgangPage.getFormularButtons();

	const vorgangBearbeiten: VorgangE2E = { ...createVorgang(), status: VorgangStatusE2E.ANGENOMMEN, name: 'DoBearbeiten' };
	const vorgangBearbeitenRevoke: VorgangE2E = { ...buildVorgang(objectIds[0], 'DoRevokeBearbeiten'), status: VorgangStatusE2E.ANGENOMMEN };

	before(() => {
		initVorgaenge([vorgangBearbeiten, vorgangBearbeitenRevoke]);

		loginAsSabine();

		waitForSpinnerToDisappear();
		exist(vorgangList.getRoot());
	})

	after(() => {
		dropCollections();
	})

	describe('bearbeiten', () => {

		it('Open Vorgang-Detail-Page', () => {
			vorgangList.getListItem(vorgangBearbeiten.name).getRoot().click();
			waitForSpinnerToDisappear();

			exist(vorgangPage.getVorgangDetailHeader().getRoot());
		})

		describe('by button', () => {

			it('should have status Angenommen', () => {
				haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[vorgangBearbeiten.status]);
			})

			it('should show snackBar', () => {
				vorgangFormularButtons.getBearbeitenButton().click();
				waitForSpinnerToDisappear();

				contains(snackBar.getMessage(), VorgangMessagesE2E.BEARBEITET);
				exist(snackBar.getCloseButton());
			})

			it('should close snackBar on close button', () => {
				snackBar.getCloseButton().click();

				notExist(snackBar.getMessage());
			})

			it('should have status In Bearbeitung', () => {
				haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.IN_BEARBEITUNG]);
			})

			it('back to vorgang list', () => {
				vorgangPage.getSubnavigation().getBackButton().click();
				waitForSpinnerToDisappear();

				exist(vorgangList.getRoot());
			})

			it('should have status In Bearbeitung', () => {
				haveText(vorgangList.getListItem(vorgangBearbeiten.name).getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.IN_BEARBEITUNG])
			})
		})
	})

	describe('bearbeiten und rückgängig machen', () => {

		it('Open Vorgang-Detail-Page', () => {
			vorgangList.getListItem(vorgangBearbeitenRevoke.name).getRoot().click();
			waitForSpinnerToDisappear();

			exist(vorgangPage.getVorgangDetailHeader().getRoot());
		})

		describe('by icon-button', () => {

			it('should have status Angenommen', () => {
				haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[vorgangBearbeitenRevoke.status]);
			})

			it('should show snackBar', () => {
				vorgangPage.getSubnavigation().getBearbeitenIconButton().click();
				waitForSpinnerToDisappear();

				exist(snackBar.getCloseButton());
				contains(snackBar.getMessage(), VorgangMessagesE2E.BEARBEITET);
			})

			it('should close snackbar on revoke', () => {
				snackBar.getRevokeButton().click();
				waitForSpinnerToDisappear();

				notExist(snackBar.getMessage());
			})

			it('should show status Angenommen', () => {
				notExist(vorgangPage.getProgressBar());

				haveText(vorgangPage.getVorgangDetailHeader().getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.ANGENOMMEN]);
			})

			it('back to vorgang list', () => {
				vorgangPage.getSubnavigation().getBackButton().click();
				waitForSpinnerToDisappear();

				exist(vorgangList.getRoot());
			})

			it('should have status Angenommen', () => {
				haveText(vorgangList.getListItem(vorgangBearbeitenRevoke.name).getStatus(), vorgangStatusLabelE2E[VorgangStatusE2E.ANGENOMMEN])
			})
		})
	})
});
