/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { KommentareInVorgangE2EComponent } from 'apps/goofy-e2e/src/components/kommentar/kommentar-list.e2e.component';
import { PostfachMailE2EComponent } from 'apps/goofy-e2e/src/components/postfach/postfach-mail.e2e.component';
import { VorgangFormularButtonsE2EComponent } from 'apps/goofy-e2e/src/components/vorgang/vorgang-formular-buttons.e2e.components';
import { VorgangSubnavigationE2EComponent } from 'apps/goofy-e2e/src/components/vorgang/vorgang-subnavigation';
import { WiedervorlagenInVorgangE2EComponent } from 'apps/goofy-e2e/src/components/wiedervorlage/wiedervorlagen-in-vorgang.e2e.component';
import { AntragstellerE2EComponent } from '../../../components/vorgang/vorgang-antragsteller.e2e.component';
import { VorgangDetailHeaderE2EComponent } from '../../../components/vorgang/vorgang-detail-header.e2e.component';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { VorgangE2E } from '../../../model/vorgang';
import { MainPage, waitForSpinnerToDisappear, waitforSpinnerToAppear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { dropCollections } from '../../../support/cypress-helper';
import { exist, notExist } from '../../../support/cypress.util';
import { UserRoleE2E, loginAsPeter, loginAsSabine } from '../../../support/user-util';
import { createVorgang, initVorgang } from '../../../support/vorgang-util';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('Vorgang Detailansicht by role', () => {
	const mainPage: MainPage = new MainPage();
	const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

	const vorgangPage: VorgangPage = new VorgangPage();
	const vorgangHeader: VorgangDetailHeaderE2EComponent = vorgangPage.getVorgangDetailHeader();
	const formularButtons: VorgangFormularButtonsE2EComponent = vorgangPage.getFormularButtons();

	const antragsteller: AntragstellerE2EComponent = vorgangPage.getAntragstellerContainer();
	const wiedervorlagenContainer: WiedervorlagenInVorgangE2EComponent = vorgangPage.getWiedervorlagenContainer();
	const vorgangSubnavigation: VorgangSubnavigationE2EComponent = vorgangPage.getSubnavigation();
	const postfachMailContainer: PostfachMailE2EComponent = vorgangPage.getPostfachMailcontainer();
	const kommentarContainer: KommentareInVorgangE2EComponent = vorgangPage.getKommentarContainer();

	const vorgang: VorgangE2E = createVorgang();

	describe('for user with role ' + UserRoleE2E.VERWALTUNG_USER, () => {

		before(() => {
			initVorgang(vorgang);

			loginAsSabine();

			waitforSpinnerToAppear();
			waitForSpinnerToDisappear();
			exist(vorgangList.getRoot());
		})

		after(() => {
			dropCollections();
		})

		describe('click on vorgang in list', () => {

			it('should show vorgang header', () => {
				vorgangList.getListItem(vorgang.name).getRoot().click();
				waitforSpinnerToAppear();
				waitForSpinnerToDisappear();

				exist(vorgangHeader.getRoot());
			})
		})

		describe('Subnavigation actions buttons', () => {

			describe('for vorgang status', () => {

				describe('annehmen', () => {

					it('should be visible', () => {
						exist(vorgangSubnavigation.getAnnehmenIconButton());
					})
				})

				describe('verwerfen', () => {

					it('should be visible', () => {
						exist(vorgangSubnavigation.getVerwerfenIconButton());
					})
				})
			})

			describe('for postfach nachricht', () => {

				it('should be visible', () => {
					exist(vorgangSubnavigation.getPostfachMailIconButton());
				})
			})

			describe('for create wiedervorlage', () => {

				it('should be visible', () => {
					exist(vorgangSubnavigation.getCreateWiedervorlageIconButton());
				})
			})
		})

		describe('Antragsteller area', () => {

			it('should be visible', () => {
				exist(antragsteller.getRoot());
			})
		})

		describe('Formular button', () => {

			describe('annehmen', () => {

				it('should be visible', () => {
					exist(formularButtons.getAnnehmenButton());
				})
			})

			describe('verwerfen', () => {

				it('should be visible', () => {
					exist(formularButtons.getVerwerfenButton());
				})
			})
		})

		describe('Wiedervorlagen area', () => {

			it('should be visible', () => {
				exist(wiedervorlagenContainer.getRoot());
			})
		})

		describe('PostfachNachricht', () => {

			it('should be visible', () => {
				exist(postfachMailContainer.getRoot());
			})
		})

		describe('Kommentar area', () => {

			it('should be visible', () => {
				exist(kommentarContainer.getRoot());
			})
		})
	})

	describe('for user with role ' + UserRoleE2E.VERWALTUNG_POSTSTELLE, () => {

		before(() => {
			initVorgang(vorgang);

			loginAsPeter();

			waitforSpinnerToAppear();
			waitForSpinnerToDisappear();
			exist(vorgangList.getRoot());
		})

		after(() => {
			dropCollections();
		})

		describe('click on vorgang in list', () => {

			it('should show vorgang header', () => {
				vorgangList.getListItem(vorgang.name).getRoot().click();
				waitforSpinnerToAppear();
				waitForSpinnerToDisappear();

				exist(vorgangHeader.getRoot());
			})
		})

		describe('Subnavigation actions buttons', () => {

			describe('for vorgang status', () => {

				describe('annehmen', () => {

					it('should NOT be visible', () => {
						notExist(vorgangSubnavigation.getAnnehmenIconButton());
					})
				})

				describe('verwerfen', () => {

					it('should NOT be visible', () => {
						notExist(vorgangSubnavigation.getVerwerfenIconButton());
					})
				})
			})

			describe('for postfach nachricht', () => {

				it('should be visible', () => {
					exist(vorgangSubnavigation.getPostfachMailIconButton());
				})
			})

			describe('for create wiedervorlage', () => {

				it('should NOT be visible', () => {
					notExist(vorgangSubnavigation.getCreateWiedervorlageIconButton());
				})
			})

			describe('for assign user', () => {

				it('should be visible', () => {
					exist(vorgangSubnavigation.getAssignUserProfileIconButton());
				})
			})
		})

		describe('Antragsteller area', () => {

			it('should be visible', () => {
				exist(antragsteller.getRoot());
			})
		})

		describe('Formular button', () => {

			describe('annehmen', () => {

				it('should NOT be visible', () => {
					notExist(formularButtons.getAnnehmenButton());
				})
			})

			describe('verwerfen', () => {

				it('should NOT be visible', () => {
					notExist(formularButtons.getVerwerfenButton());
				})
			})
		})

		describe('Wiedervorlagen area', () => {

			it('should NOT be visible', () => {
				notExist(wiedervorlagenContainer.getRoot());
			})
		})

		describe('PostfachNachricht', () => {

			it('should be visible', () => {
				exist(postfachMailContainer.getRoot());
			})
		})

		describe('Kommentar area', () => {

			it('should be visible', () => {
				exist(kommentarContainer.getRoot());
			})
		})
	})
});