/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { VorgangAttachedItemE2E } from 'apps/goofy-e2e/src/model/vorgang-attached-item';
import { createDateInTheFuture, createDateInThePast, createDateToday } from 'apps/goofy-e2e/src/support/tech.util';
import { initVorgangAttachedItem } from 'apps/goofy-e2e/src/support/vorgang-attached-item-util';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { WiedervorlageInVorgangE2EComponent } from '../../../components/wiedervorlage/wiedervorlage-in-vorgang.e2e.component';
import { WiedervorlagenInVorgangE2EComponent } from '../../../components/wiedervorlage/wiedervorlagen-in-vorgang.e2e.component';
import { VorgangE2E } from '../../../model/vorgang';
import { WiedervorlageE2E } from '../../../model/wiedervorlage';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { dropCollections } from '../../../support/cypress-helper';
import { containClass, exist, notContainClass, notExist } from '../../../support/cypress.util';
import { loginAsSabine } from '../../../support/user-util';
import { createVorgang, initVorgang, objectIds } from '../../../support/vorgang-util';
import { createWiedervorlageAttachedItem, createWiedervorlageItem } from '../../../support/wiedervorlage-util';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('Vorgang Wiedervorlagen List', () => {
	const mainPage: MainPage = new MainPage();
	const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

	const vorgangPage: VorgangPage = new VorgangPage();
	const wiedervorlageInVorgangContainer: WiedervorlagenInVorgangE2EComponent = vorgangPage.getWiedervorlagenContainer();

	const vorgang: VorgangE2E = createVorgang();

	const wiedervorlageFristToday: WiedervorlageE2E = { ...createWiedervorlageItem('WiedervorlageFristToday'), frist: createDateToday() };
	const vorgangAttachedItemWithWiedervorlageFristToday: VorgangAttachedItemE2E = { ...createWiedervorlageAttachedItem(objectIds[0], vorgang._id.$oid), item: wiedervorlageFristToday };


	const wiedervorlageFristInThePast: WiedervorlageE2E = { ...createWiedervorlageItem('WiedervorlageFristInThePast'), frist: createDateInThePast() };
	const vorgangAttachedItemWithWiedervorlageFristInThePast: VorgangAttachedItemE2E = { ...createWiedervorlageAttachedItem(objectIds[1], vorgang._id.$oid), item: wiedervorlageFristInThePast };

	const wiedervorlageFristInTheFuture: WiedervorlageE2E = { ...createWiedervorlageItem('WiedervorlageFristInTheFuture'), frist: createDateInTheFuture() };
	const vorgangAttachedItemWithWiedervorlageFristInThFuture: VorgangAttachedItemE2E = { ...createWiedervorlageAttachedItem(objectIds[2], vorgang._id.$oid), item: wiedervorlageFristInTheFuture };

	const wiedervorlageFristIsDone: WiedervorlageE2E = { ...createWiedervorlageItem('WiedervorlageFristIsDone'), done: true };
	const vorgangAttachedItemWithWiedervorlageIsDone: VorgangAttachedItemE2E = { ...createWiedervorlageAttachedItem(objectIds[3], vorgang._id.$oid), item: wiedervorlageFristIsDone };

	const wiedervorlageWithoutDetails: WiedervorlageE2E = { ...createWiedervorlageItem('WiedervorlageWithoutDetail'), beschreibung: '' };
	const vorgangAttachedItemWithWiedervorlageWithoutDetails: VorgangAttachedItemE2E = { ...createWiedervorlageAttachedItem(objectIds[4], vorgang._id.$oid), item: wiedervorlageWithoutDetails };

	before(() => {
		initVorgang(vorgang);

		initVorgangAttachedItem([vorgangAttachedItemWithWiedervorlageFristToday, vorgangAttachedItemWithWiedervorlageFristInThePast, vorgangAttachedItemWithWiedervorlageFristInThFuture, vorgangAttachedItemWithWiedervorlageIsDone, vorgangAttachedItemWithWiedervorlageWithoutDetails]);

		loginAsSabine();

		waitForSpinnerToDisappear();
		exist(vorgangList.getRoot());
	})

	after(() => {
		dropCollections();
	})

	it('navigate to vorgang detail', () => {
		vorgangList.getListItem(vorgang.name).getRoot().click();
		waitForSpinnerToDisappear();

		exist(vorgangPage.getVorgangDetailHeader().getRoot());
	})

	describe('check status on wiedervorlage', () => {

		it('should show status yellow on frist today', () => {
			const wiedervorlage: WiedervorlageInVorgangE2EComponent = wiedervorlageInVorgangContainer.getWiedervorlage(wiedervorlageFristToday.betreff);

			exist(wiedervorlage.getRoot());
			notContainClass(wiedervorlage.getStatus().getStatusDot(), 'red');
			containClass(wiedervorlage.getStatus().getStatusDot(), 'yellow');

			exist(wiedervorlage.getExpandButton());
		})

		it('should show status red on frist in the past', () => {
			const wiedervorlage: WiedervorlageInVorgangE2EComponent = wiedervorlageInVorgangContainer.getWiedervorlage(wiedervorlageFristInThePast.betreff);

			exist(wiedervorlage.getRoot());
			containClass(wiedervorlage.getStatus().getStatusDot(), 'red');
			notContainClass(wiedervorlage.getStatus().getStatusDot(), 'yellow');

			exist(wiedervorlage.getExpandButton());
		})

		it('should show status grey on frist in the future', () => {
			const wiedervorlage: WiedervorlageInVorgangE2EComponent = wiedervorlageInVorgangContainer.getWiedervorlage(wiedervorlageFristInTheFuture.betreff);

			exist(wiedervorlage.getRoot());
			notContainClass(wiedervorlage.getStatus().getStatusDot(), 'red');
			notContainClass(wiedervorlage.getStatus().getStatusDot(), 'yellow');

			exist(wiedervorlage.getExpandButton());
		})

		it('should show status white with check on frist is done', () => {
			const wiedervorlage: WiedervorlageInVorgangE2EComponent = wiedervorlageInVorgangContainer.getWiedervorlage(wiedervorlageFristIsDone.betreff);

			exist(wiedervorlage.getRoot());
			containClass(wiedervorlage.getStatus().getStatusDot(), 'erledigt');

			exist(wiedervorlage.getExpandButton());
		})
	})

	describe('on wiedervorlage with detailinformation', () => {

		it('should show expand button', () => {
			const wiedervorlage: WiedervorlageInVorgangE2EComponent = wiedervorlageInVorgangContainer.getWiedervorlage(wiedervorlageFristToday.betreff);

			exist(wiedervorlage.getRoot());
			exist(wiedervorlage.getExpandButton());
		})

		it('should show detail information', () => {
			const wiedervorlage: WiedervorlageInVorgangE2EComponent = wiedervorlageInVorgangContainer.getWiedervorlage(wiedervorlageFristToday.betreff);

			exist(wiedervorlage.getRoot());
			exist(wiedervorlage.getExpandButton());

			wiedervorlage.expandItem();

			exist(wiedervorlage.getExpandable());
		})
	})

	describe('on wiedervorlage without detailinformation', () => {

		it('should not show expand button for details', () => {
			const wiedervorlage: WiedervorlageInVorgangE2EComponent = vorgangPage.getWiedervorlagenContainer().getWiedervorlage(wiedervorlageWithoutDetails.betreff);

			exist(wiedervorlage.getRoot());
			notExist(wiedervorlage.getExpandButton());
		})
	})
});