/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { registerLocaleData } from '@angular/common';
import { HttpClientModule } from '@angular/common/http';
import localeDe from '@angular/common/locales/de';
import { LOCALE_ID, NgModule } from '@angular/core';
import { BrowserModule } from '@angular/platform-browser';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { RouterModule, Routes, UrlSerializer } from '@angular/router';
import { ApiRootModule } from '@goofy-client/api-root-shared';
import { AppSharedModule } from '@goofy-client/app-shared';
import { EnvironmentModule } from '@goofy-client/environment-shared';
import { HintSharedModule } from '@goofy-client/hint-shared';
import { NavigationModule } from '@goofy-client/navigation';
import { OzgCloudUrlSerializer } from '@goofy-client/navigation-shared';
import { UiModule } from '@goofy-client/ui';
import { EffectsModule } from '@ngrx/effects';
import { StoreRouterConnectingModule } from '@ngrx/router-store';
import { StoreModule } from '@ngrx/store';
import { StoreDevtoolsModule } from '@ngrx/store-devtools';
import { OAuthModule } from 'angular-oauth2-oidc';
import { environment } from '../environments/environment';
import { AppComponent } from './app.component';

registerLocaleData(localeDe);

const routes: Routes = [
	{
		path: '',
		loadChildren: () => import('@goofy-client/vorgang').then(m => m.VorgangModule)
	}
];

@NgModule({
	declarations: [
		AppComponent,
	],
	imports: [
		BrowserModule,
		HttpClientModule,
		BrowserAnimationsModule,
		RouterModule.forRoot(routes, {
			initialNavigation: 'disabled',
			onSameUrlNavigation: 'reload',

		}),
		UiModule,
		EnvironmentModule,
		OAuthModule.forRoot({
			resourceServer: {
				sendAccessToken: true
			},
		}),
		ApiRootModule,
		NavigationModule,
		AppSharedModule,
		StoreModule.forRoot({}, {
			metaReducers: [],
			runtimeChecks: {
				strictActionImmutability: true,
				strictStateImmutability: true
			}
		}),
		EffectsModule.forRoot([]),
		environment.production ? [] : StoreDevtoolsModule.instrument(),
		StoreRouterConnectingModule.forRoot(),

		// Workaround: load any module depending on ApiRootActions.loadApiRootSuccess
		// TODO Check if a future NgRx version will solve Lazy Loading + Effects.forFeature()
		HintSharedModule
		//
	],
	providers: [
		{ provide: LOCALE_ID, useValue: 'de' },
		{ provide: UrlSerializer, useClass: OzgCloudUrlSerializer }
	],
	bootstrap: [AppComponent]
})
export class AppModule { }
