import { CommandProps, CommandResource, CreateCommandProps } from '@goofy-client/command-shared';
import { StateResource, createEmptyStateResource, createStateResource } from '@goofy-client/tech-shared';
import { Action, ActionReducer, createReducer, on } from '@ngrx/store';
import { isCreateBescheidCommand } from '../bescheid.util';

import * as CommandActions from '../../../../command-shared/src/lib/+state/command.actions';

export const BESCHEID_FEATURE_KEY = 'BescheidState';

export interface BescheidPartialState {
	readonly [BESCHEID_FEATURE_KEY]: BescheidState;
}

export interface BescheidState {
	bescheidCommand: StateResource<CommandResource>
}

export const initialState: BescheidState = {
	bescheidCommand: createEmptyStateResource()
};

const bescheidReducer: ActionReducer<BescheidState, Action> = createReducer(
	initialState,
	on(CommandActions.createCommand, (state: BescheidState, props: CreateCommandProps): BescheidState => {
		return isCreateBescheidCommand(props.command.order)
		? { ...state, bescheidCommand: { ...state.bescheidCommand, loading: true } }
		: state;
	}),
	on(CommandActions.createCommandSuccess, (state: BescheidState, props: CommandProps): BescheidState => {
		return isCreateBescheidCommand(props.command.order)
		? { ...state, bescheidCommand: createStateResource(props.command) }
		: state;
	}),
);

export function reducer(state: BescheidState, action: Action): BescheidState {
	return bescheidReducer(state, action);
}
