/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatIcon } from '@angular/material/icon';
import { ForwardingLinkRel, ForwardingResource, ForwardingService } from '@goofy-client/forwarding-shared';
import { HasLinkPipe } from '@goofy-client/tech-shared';
import { mock } from '@goofy-client/test-utils';
import { OzgcloudStrokedButtonWithSpinnerComponent } from '@goofy-client/ui';
import { createForwardingResource } from 'libs/forwarding-shared/test/forwarding';
import { MockComponent } from 'ng-mocks';
import { VorgangForwardZustellenContainerComponent } from './vorgang-forward-zustellen-container.component';

describe('VorgangForwardZustellenContainerComponent', () => {
	let component: VorgangForwardZustellenContainerComponent;
	let fixture: ComponentFixture<VorgangForwardZustellenContainerComponent>;

	const forwarding: ForwardingResource = createForwardingResource();

	const markAsSuccess: string = '[data-test-id="mark-as-success-button"]';
	const markAsFail: string = '[data-test-id="mark-as-fail-button"]';

	const failedMessage: string = '[data-test-id="failed-message"]';
	const successfullMessage: string = '[data-test-id="successfull-message"]';

	const forwardingService = mock(ForwardingService);

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				HasLinkPipe,
				VorgangForwardZustellenContainerComponent,
				MatIcon,
				MockComponent(OzgcloudStrokedButtonWithSpinnerComponent)
			],
			providers: [
				{
					provide: ForwardingService,
					useValue: forwardingService
				}
			]
		})
	})

	beforeEach(() => {
		fixture = TestBed.createComponent(VorgangForwardZustellenContainerComponent);
		component = fixture.componentInstance;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	it('should call service mark as success', () => {
		component.forwarding = forwarding;
		fixture.detectChanges();

		component.markAsSuccess();

		expect(forwardingService.markAsSuccess).toHaveBeenCalledWith(forwarding);
	})

	it('should call service mark as fail', () => {
		component.forwarding = forwarding;
		fixture.detectChanges();

		component.markAsFail();

		expect(forwardingService.markAsFail).toHaveBeenCalledWith(forwarding);
	})

	describe('mark as success button', () => {

		it('should be visible', () => {
			component.forwarding = createForwardingResource([ForwardingLinkRel.MARK_AS_SUCCESS]);

			fixture.detectChanges();
			const button = fixture.nativeElement.querySelector(markAsSuccess);

			expect(button).toBeInstanceOf(HTMLElement);
		})

		it('should be hidden', () => {
			component.forwarding = forwarding;

			fixture.detectChanges();
			const button = fixture.nativeElement.querySelector(markAsSuccess);

			expect(button).not.toBeInstanceOf(HTMLElement);
		})
	})

	describe('mark as fail button', () => {

		it('should be visible', () => {
			component.forwarding = createForwardingResource([ForwardingLinkRel.MARK_AS_FAIL]);

			fixture.detectChanges();
			const button = fixture.nativeElement.querySelector(markAsFail);

			expect(button).toBeInstanceOf(HTMLElement);
		})

		it('should be hidden', () => {
			component.forwarding = forwarding;

			fixture.detectChanges();
			const button = fixture.nativeElement.querySelector(markAsFail);

			expect(button).not.toBeInstanceOf(HTMLElement);
		})
	})

	describe('failed message', () => {

		it('should be visible', () => {
			component.forwarding = createForwardingResource([ForwardingLinkRel.FAILED]);

			fixture.detectChanges();
			const button = fixture.nativeElement.querySelector(failedMessage);

			expect(button).toBeInstanceOf(HTMLElement);
		})

		it('should be hidden', () => {
			component.forwarding = forwarding;

			fixture.detectChanges();
			const button = fixture.nativeElement.querySelector(failedMessage);

			expect(button).not.toBeInstanceOf(HTMLElement);
		})
	})

	describe('successfull message', () => {

		it('should be visible', () => {
			component.forwarding = createForwardingResource([ForwardingLinkRel.SUCCESSFUL]);

			fixture.detectChanges();
			const button = fixture.nativeElement.querySelector(successfullMessage);

			expect(button).toBeInstanceOf(HTMLElement);
		})

		it('should be hidden', () => {
			component.forwarding = forwarding;

			fixture.detectChanges();
			const button = fixture.nativeElement.querySelector(successfullMessage);

			expect(button).not.toBeInstanceOf(HTMLElement);
		})
	})
});
