import { ApiError, ApiErrorAction, createStateResource } from '@goofy-client/tech-shared';
import { Action } from '@ngrx/store';
import { createApiError } from '../../../../tech-shared/test/error';
import { createHintListResource } from '../../../test/hint';
import { HintListResource } from '../hint.model';
import { HintListProps } from './hint.actions';
import { HintState, initialState, reducer } from './hint.reducer';

import * as HintActions from './hint.actions';

describe('HintReducer', () => {

	describe('unknown action', () => {

		it('should return current state', () => {
			const action = {} as Action;

			const result = reducer(initialState, action);

			expect(result).toBe(initialState);
		})
	})

	describe('on loadHintListSuccess action', () => {

		it('should set loaded resource into state', () => {
			const hintList: HintListResource = createHintListResource();
			const action = HintActions.loadHintListSuccess(<HintListProps>{ hintList });

			const result: HintState = reducer(initialState, action);

			expect(result.hintList).toEqual(createStateResource(hintList));
		})
	})

	describe('on loadHintListFailed action', () => {

		it('should set error into state', () => {
			const apiError: ApiError = createApiError();
			const action = HintActions.loadHintListFailed(<ApiErrorAction>{ apiError });

			const result: HintState = reducer(initialState, action);

			expect(result.hintList.error).toBe(apiError);
		})
	})
})