import { ApiRootLinkRel, ApiRootResource } from '@goofy-client/api-root-shared';
import { mock, useFromMock } from '@goofy-client/test-utils';
import { ResourceFactory } from '@ngxp/rest';
import { cold, hot } from 'jest-marbles';
import { createHintListResource } from 'libs/hint-shared/test/hint';
import { createApiRootResource } from '../../../../../libs/api-root-shared/test/api-root';
import { HintListResource } from '../hint.model';
import { HintRepository } from './hint.repository';

describe('HintRepository', () => {

	let service: HintRepository;
	let resourceFactory = mock(ResourceFactory);
	let resourceWrapper = { get: jest.fn() };

	beforeEach(() => {
		service = new HintRepository(useFromMock(resourceFactory));

		resourceFactory.from.mockReturnValue(resourceWrapper);
	})

	describe('loadAll', () => {
		const hintListResource: HintListResource = createHintListResource();
		const apiRoot: ApiRootResource = createApiRootResource();

		beforeEach(() => {
			resourceWrapper.get.mockReturnValue(hot('a', { a: hintListResource }));
		})

		it('should call resourceFactory with given resource', () => {
			service.loadAll(apiRoot);

			expect(resourceFactory.from).toHaveBeenCalledWith(apiRoot);
		})

		it('should call resourceWrapper with link', () => {
			service.loadAll(apiRoot);

			expect(resourceWrapper.get).toHaveBeenCalledWith(ApiRootLinkRel.HINTS);
		})

		it('should return result', () => {
			const result = service.loadAll(apiRoot);

			expect(result).not.toBeNull();
			expect(result).toBeObservable(cold('a', { a: hintListResource }));
		})
	})
})