import { mock, useFromMock } from '@goofy-client/test-utils';
import { VorgangWithEingangLinkRel, VorgangWithEingangResource } from '@goofy-client/vorgang-shared';
import { ResourceFactory } from '@ngxp/rest';
import { cold, hot } from 'jest-marbles';
import { createLoeschAnforderungResource } from 'libs/loesch-anforderung-shared/test/loesch-anforderung';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { LoeschAnforderungResource } from '../loesch-anforderung.model';
import { LoeschAnforderungRepository } from './loesch-anforderung.repository';

describe('LoeschAnforderungRepository', () => {
	let repository: LoeschAnforderungRepository;
	let resourceFactory = mock(ResourceFactory);
	let resourceWrapper = { get: jest.fn() };

	beforeEach(() => {
		repository = new LoeschAnforderungRepository(useFromMock(resourceFactory));

		resourceFactory.fromId.mockReturnValue(resourceWrapper);
		resourceFactory.from.mockReturnValue(resourceWrapper);
	})

	describe('getLoeschAnforderung', () => {

		const vorgangWithEingangResource: VorgangWithEingangResource = createVorgangWithEingangResource();
		const loeschAnforderungResource: LoeschAnforderungResource = createLoeschAnforderungResource()

		beforeEach(() => {
			resourceWrapper.get.mockReturnValue(hot('a', { a: loeschAnforderungResource }));
		})

		it('should call resourceFactory with resource', () => {
			repository.getLoeschAnforderung(vorgangWithEingangResource);

			expect(resourceFactory.from).toHaveBeenCalledWith(vorgangWithEingangResource);
		})

		it('should call resourceWrapper with link', () => {
			repository.getLoeschAnforderung(vorgangWithEingangResource);

			expect(resourceWrapper.get).toHaveBeenCalledWith(VorgangWithEingangLinkRel.LOESCH_ANFORDERUNG)
		})

		it('should return result', () => {
			let result = repository.getLoeschAnforderung(vorgangWithEingangResource);

			expect(result).not.toBeNull();
			expect(result).toBeObservable(cold('a', { a: loeschAnforderungResource }));
		})
	})
})