import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatDialogModule, MatDialogRef } from '@angular/material/dialog';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { LoeschAnforderungResource, LoeschAnforderungService } from '@goofy-client/loesch-anforderung-shared';
import { HasLinkPipe, StateResource, createEmptyStateResource, createStateResource } from '@goofy-client/tech-shared';
import { mock } from '@goofy-client/test-utils';
import { BasicDialogComponent, BasicDialogData, DialogService } from '@goofy-client/ui';
import { VorgangService } from '@goofy-client/vorgang-shared';
import { createCommandResource } from 'libs/command-shared/test/command';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { EndgueltigLoeschenButtonContainerComponent } from './endgueltig-loeschen-button-container.component';
import { EndgueltigLoeschenButtonComponent } from './endgueltig-loeschen-button/endgueltig-loeschen-button.component';

describe('EndgueltigLoeschenButtonContainerComponent', () => {
	let component: EndgueltigLoeschenButtonContainerComponent;
	let fixture: ComponentFixture<EndgueltigLoeschenButtonContainerComponent>;

	const loeschAnforderungResource: StateResource<LoeschAnforderungResource> = createEmptyStateResource();

	const dialogRef = <MatDialogRef<BasicDialogComponent>>{};
	dialogRef.afterClosed = jest.fn().mockReturnValue(of(true));

	const loeschAnforderungService = mock(LoeschAnforderungService);
	const vorgangService = mock(VorgangService);
	const dialogService = mock(DialogService);

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				EndgueltigLoeschenButtonContainerComponent,
				HasLinkPipe,
				MockComponent(EndgueltigLoeschenButtonComponent)
			],
			imports: [
				BrowserAnimationsModule,
				MatDialogModule
			],
			providers: [
				{
					provide: MatDialogRef,
					useValue: {}
				},
				{
					provide: LoeschAnforderungService,
					useValue: loeschAnforderungService
				},
				{
					provide: VorgangService,
					useValue: vorgangService,
				},
				{
					provide: DialogService,
					useValue: dialogService,
				}
			]
		}).compileComponents();

		fixture = TestBed.createComponent(EndgueltigLoeschenButtonContainerComponent);
		component = fixture.componentInstance;
		loeschAnforderungService.getEndgueltigLoeschenCommand.mockReturnValue(of(createStateResource(createCommandResource())));
		dialogService.openBasic = jest.fn().mockReturnValue(dialogRef);
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('ngOnInit', () => {

		it('should call vorgangCommandService#getEndgueltigLoeschenCommand', () => {
			component.ngOnInit();

			expect(loeschAnforderungService.getEndgueltigLoeschenCommand).toHaveBeenCalled();
		})

		it('should call loeschAnforderungService#getLoeschAnforderung', () => {
			component.ngOnInit();

			expect(loeschAnforderungService.getLoeschAnforderung).toHaveBeenCalled();
		})
	})

	describe('openEndgueltigLoeschenDialog', () => {

		it('should call dialogService.openBasic', () => {
			component.openEndgueltigLoeschenDialog(loeschAnforderungResource);

			expect(dialogService.openBasic).toHaveBeenCalled();
		})

		it('should call createBasicDialogConfig', () => {
			component.createBasicDialogConfig = jest.fn();

			component.openEndgueltigLoeschenDialog(loeschAnforderungResource);

			expect(component.createBasicDialogConfig).toHaveBeenCalled();
		})

		it('should call dialogRef.afterClosed', () => {
			component.openEndgueltigLoeschenDialog(loeschAnforderungResource);

			expect(dialogRef.afterClosed).toHaveBeenCalled();
		})

		it('should call component.endgueltigLoeschen', () => {
			component.endgueltigLoeschen = jest.fn();

			component.openEndgueltigLoeschenDialog(loeschAnforderungResource);

			expect(component.endgueltigLoeschen).toHaveBeenCalled();
		})

	})

	describe('createBasicDialogConfig', () => {

		it('should return BasicDialogData', () => {
			const basicDialogData: BasicDialogData = {
				title: 'Möchten Sie den Vorgang endgültig löschen?',
				message: 'Diese Aktion kann nicht widerrufen werden.',
				okText: 'Endgültig löschen',
				okSvgIcon: 'delete_finally'
			}

			const res = component.createBasicDialogConfig();

			expect(res).toMatchObject<BasicDialogData>(basicDialogData);
		})
	})

	describe('endgueltigLoeschen', () => {

		it('should call service', () => {
			const spy = jest.spyOn(loeschAnforderungService, 'endgueltigLoeschen');

			component.endgueltigLoeschen(loeschAnforderungResource);

			expect(spy).toHaveBeenCalledWith(loeschAnforderungResource.resource);
		})
	})
});
