import { ComponentFixture, TestBed } from '@angular/core/testing';
import { BinaryFileAttachmentContainerComponent } from '@goofy-client/binary-file';
import { BinaryFileResource } from '@goofy-client/binary-file-shared';
import { PostfachMailListLinkRel, PostfachMailListResource, PostfachService } from '@goofy-client/postfach-shared';
import { StateResource, createStateResource } from '@goofy-client/tech-shared';
import { Mock, getMockComponent, mock } from '@goofy-client/test-utils';
import { createBinaryFileResource } from 'libs/binary-file-shared/test/binary-file';
import { PostfachTestFactory, createPostfachMailListResource } from 'libs/postfach-shared/test/postfach';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { PostfachMailFormservice } from '../postfach-mail.formservice';
import { PostfachNachrichtAttachmentContainerComponent } from './postfach-nachricht-attachment-container.component';

describe('PostfachNachrichtAttachmentContainerComponent', () => {
	let component: PostfachNachrichtAttachmentContainerComponent;
	let fixture: ComponentFixture<PostfachNachrichtAttachmentContainerComponent>;

	const service: Mock<PostfachService> = mock(PostfachService);

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				PostfachNachrichtAttachmentContainerComponent,
				MockComponent(BinaryFileAttachmentContainerComponent)
			],
			providers: [
				{
					provide: PostfachService,
					useValue: service
				}
			]
		}).compileComponents();

		fixture = TestBed.createComponent(PostfachNachrichtAttachmentContainerComponent);
		component = fixture.componentInstance;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('ngOnInit', () => {

		beforeEach(() => {
			component.initAttachments = jest.fn();
		})

		it('should get list stateResource', () => {
			component.ngOnInit();

			expect(service.getPostfachMailListByVorgang).toHaveBeenCalled();
		})

		it('should initAttachments on existing postfachNachricht', () => {
			component.postfachNachricht = PostfachTestFactory.POSTFACH_NACHRICHT_RESOURCE;

			component.ngOnInit();

			expect(component.initAttachments).toHaveBeenCalled();
		})

		it('should not init attachments on missing postfachNachricht', () => {
			component.postfachNachricht = undefined;

			component.ngOnInit();

			expect(component.initAttachments).not.toHaveBeenCalled();
		})
	})

	describe('binary file attachment container', () => {

		it('should be called with formArrayName', () => {
			const binaryFileAttachmentContainer: BinaryFileAttachmentContainerComponent = getMockComponent(fixture, BinaryFileAttachmentContainerComponent);
			expect(binaryFileAttachmentContainer.formArrayName).toBe(PostfachMailFormservice.FIELD_ATTACHMENTS);
		})

		it('should be called with existFiles', () => {
			const attachments: BinaryFileResource[] = [createBinaryFileResource()];
			component.attachments$ = of(attachments);

			fixture.detectChanges();

			const binaryFileAttachmentContainer: BinaryFileAttachmentContainerComponent = getMockComponent(fixture, BinaryFileAttachmentContainerComponent);
			expect(binaryFileAttachmentContainer.existFiles).toBe(attachments);
		})

		it('should be called with upload stateResource', () => {
			const listStateResource: StateResource<PostfachMailListResource> = createStateResource(createPostfachMailListResource());
			component.postfachNachrichtListStateResource$ = of(listStateResource);

			fixture.detectChanges();

			const binaryFileAttachmentContainer: BinaryFileAttachmentContainerComponent = getMockComponent(fixture, BinaryFileAttachmentContainerComponent);
			expect(binaryFileAttachmentContainer.uploadStateResource).toBe(listStateResource);
		})

		it('should be called with linkRel upload attachment', () => {
			const binaryFileAttachmentContainer: BinaryFileAttachmentContainerComponent = getMockComponent(fixture, BinaryFileAttachmentContainerComponent);
			expect(binaryFileAttachmentContainer.linkRelUploadAttachment).toBe(PostfachMailListLinkRel.UPLOAD_ATTACHMENT);
		})
	})

	describe('ngOnDestroy', () => {

		it('should clear attachments', () => {
			component.ngOnDestroy();

			expect(service.clearAttachmentList).toHaveBeenCalled();
		})
	})
});
