import { ComponentFixture, TestBed } from '@angular/core/testing';
import { PostfachMailFormDialogData, PostfachMailResource } from '@goofy-client/postfach-shared';
import { EMPTY_STRING, createStateResource } from '@goofy-client/tech-shared';
import { Mock, dispatchEventFromFixture, getMockComponent, mock } from '@goofy-client/test-utils';
import { DialogService, OzgcloudIconComponent } from '@goofy-client/ui';
import { Antragsteller, VorgangWithEingangResource } from '@goofy-client/vorgang-shared';
import { createPostfachMailResource } from 'libs/postfach-shared/test/postfach';
import { PostfachMailFormComponent } from 'libs/postfach/src/lib/postfach-mail-form/postfach-mail-form.component';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { createAntragsteller, createEingang, createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { PostfachNachrichtEditButtonContainerComponent } from './postfach-nachricht-edit-button-container.component';

describe('PostfachNachrichtEditButtonContainerComponent', () => {
	let component: PostfachNachrichtEditButtonContainerComponent;
	let fixture: ComponentFixture<PostfachNachrichtEditButtonContainerComponent>;

	const postfachNachrichtEditButton: string = getDataTestIdOf('postfach-nachricht-edit-button');

	const dialogService: Mock<DialogService> = mock(DialogService);

	const postfachNachricht: PostfachMailResource = createPostfachMailResource();

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				PostfachNachrichtEditButtonContainerComponent,
				MockComponent(OzgcloudIconComponent)
			],
			providers:[
				{
					provide: DialogService,
					useValue: dialogService
				}
			]
		}).compileComponents();

		fixture = TestBed.createComponent(PostfachNachrichtEditButtonContainerComponent);
		component = fixture.componentInstance;
		component.postfachNachricht = postfachNachricht;
		component.vorgangStateResource = createStateResource(createVorgangWithEingangResource());
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('ozgcloud icon component', () => {

		it('should be called with icon', () => {
			const ozgcloudIconComponent: OzgcloudIconComponent = getMockComponent(fixture, OzgcloudIconComponent);

			expect(ozgcloudIconComponent.icon).toBe('edit');
		})
	})

	describe('on click on button', () => {

		it('should call dialogService', () => {
			dispatchEventFromFixture(fixture, postfachNachrichtEditButton, 'click');

			expect(dialogService.openFixed).toHaveBeenCalled();
		})

		it('should build dialogData', () => {
			component.buildDialogData = jest.fn();

			dispatchEventFromFixture(fixture, postfachNachrichtEditButton, 'click');

			expect(component.buildDialogData).toHaveBeenCalled();
		})
	})

	describe('buildDialogData', () => {

		it('should have title', () => {
			const dialogData: PostfachMailFormDialogData = component.buildDialogData();

			expect(dialogData.title).toBe('Nachricht bearbeiten');
		})

		it('should have component', () => {
			const dialogData: PostfachMailFormDialogData = component.buildDialogData();

			expect(dialogData.component).toBe(PostfachMailFormComponent);
		})

		it('should have postfachNachricht', () => {
			const dialogData: PostfachMailFormDialogData = component.buildDialogData();

			expect(dialogData.postfachNachricht).toBe(postfachNachricht);
		})

		it('should get Empfaenger', () => {
			component.getEmpfaenger = jest.fn();

			component.buildDialogData();

			expect(component.getEmpfaenger).toHaveBeenCalled();
		})
	})

	describe('getEmpfaenger', () => {

		it('should return nachname only if exists', () => {
			const antragsteller: Antragsteller = { ...createAntragsteller(), vorname: undefined };
			component.vorgangStateResource = createStateResource(createWithAntragsteller(antragsteller));

			const empfaenger: string = component.getEmpfaenger();

			expect(empfaenger).toEqual(antragsteller.nachname);
		})

		it('should return vorname only if exists', ()=> {
			const antragsteller: Antragsteller = { ...createAntragsteller(), nachname: undefined };
			component.vorgangStateResource = createStateResource(createWithAntragsteller(antragsteller));

			const empfaenger: string = component.getEmpfaenger();

			expect(empfaenger).toEqual(antragsteller.vorname);
		})

		it('should return name and vorname if exists', () => {
			const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource();
			component.vorgangStateResource = createStateResource(vorgangWithEingang);

			const empfaenger: string = component.getEmpfaenger();

			expect(empfaenger).toEqual(`${vorgangWithEingang.eingang.antragsteller.vorname} ${vorgangWithEingang.eingang.antragsteller.nachname}`);
		})

		it('should return empty string if none exists', () => {
			const antragsteller: Antragsteller = { ...createAntragsteller(), nachname: undefined, vorname: undefined };
			component.vorgangStateResource = createStateResource(createWithAntragsteller(antragsteller));

			const empfaenger: string = component.getEmpfaenger();

			expect(empfaenger).toEqual(EMPTY_STRING);
		})

		function createWithAntragsteller(antragsteller: Antragsteller): VorgangWithEingangResource {
			return { ...createVorgangWithEingangResource(), eingang: { ...createEingang(), antragsteller } };
		}
	})
});
