/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { BinaryFileListResource } from '@goofy-client/binary-file-shared';
import { PostfachMailResource, PostfachService } from '@goofy-client/postfach-shared';
import { ConvertForDataTestPipe, createStateResource } from '@goofy-client/tech-shared';
import { Mock, mock } from '@goofy-client/test-utils';
import { SpinnerComponent } from '@goofy-client/ui';
import { createBinaryFileListResource } from 'libs/binary-file-shared/test/binary-file';
import { createPostfachMailResource } from 'libs/postfach-shared/test/postfach';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { PostfachMailAttachmentsComponent } from './postfach-mail-attachments.component';
import { VerticalBinaryFileListComponent } from '@goofy-client/binary-file';

registerLocaleData(localeDe);

describe('PostfachMailAttachmentsComponent', () => {
	let component: PostfachMailAttachmentsComponent;
	let fixture: ComponentFixture<PostfachMailAttachmentsComponent>;

	const postfachNachricht: PostfachMailResource = createPostfachMailResource();

	const postfachService: Mock<PostfachService> = mock(PostfachService);

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				PostfachMailAttachmentsComponent,
				ConvertForDataTestPipe,
				MockComponent(VerticalBinaryFileListComponent),
				MockComponent(SpinnerComponent)
			],
			providers: [{
				provide: PostfachService,
				useValue: postfachService
			}]
		}).compileComponents();
	});

	beforeEach(() => {
		fixture = TestBed.createComponent(PostfachMailAttachmentsComponent);
		component = fixture.componentInstance;
		component.postfachNachricht = postfachNachricht;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('ngOnChanges', () => {

		const attachmentList: BinaryFileListResource = createBinaryFileListResource();

		beforeEach(() => {
			postfachService.loadAttachments.mockReturnValue(of(createStateResource(attachmentList)));
		})

		it('should call service', () => {
			component.ngOnChanges();

			expect(postfachService.loadAttachments).toHaveBeenCalledWith(postfachNachricht);
		})
	})
});