/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { UntypedFormBuilder, UntypedFormControl, UntypedFormGroup } from '@angular/forms';
import { CommandResource } from '@goofy-client/command-shared';
import { ApiError, createEmptyStateResource, createErrorStateResource, StateResource } from '@goofy-client/tech-shared';
import { Resource } from '@ngxp/rest';
import { createApiError } from 'libs/tech-shared/test/error';
import { Observable } from 'rxjs';
import { AbstractFormService } from './formservice.abstract';

describe('AbstractFormService', () => {
	let formService: AbstractFormService;

	beforeEach(() => {
		formService = new DummyFormService(new UntypedFormBuilder());
	})

	it('should create', () => {
		expect(formService).toBeTruthy();
	});

	describe('handleResponse', () => {
		const apiError: ApiError = createApiError();
		const stateResource: StateResource<CommandResource> = createErrorStateResource(apiError);

		beforeEach(() => {
			formService.setError = jest.fn();
		})

		it('should setError on validation error', () => {
			formService.handleResponse({ ...stateResource, loading: false });

			expect(formService.setError).toHaveBeenCalledWith(apiError);
		})

		it('should return stateresource while loading', () => {
			const commandStateResource: StateResource<CommandResource> = createEmptyStateResource(true);

			const result: StateResource<Resource> = formService.handleResponse(commandStateResource);

			expect(result).toBe(commandStateResource);
		})
	})

	describe('patch', () => {

		it('should set form value', () => {
			const formValue = { [DummyFormService.FIELD]: 'huhu' };

			formService.patch(formValue);

			expect(formService.form.value).toEqual(formValue);
		})
	})
})


export class DummyFormService extends AbstractFormService {

	static FIELD = 'field';

	constructor(formBuilder: UntypedFormBuilder) {
		super(formBuilder);
	}

	initForm(): UntypedFormGroup {
		return this.formBuilder.group({
			[DummyFormService.FIELD]: new UntypedFormControl(null)
		});
	}

	doSubmit(): Observable<StateResource<any>> {
		throw new Error('Method not implemented.');
	}

	getPathPrefix(): string {
		throw new Error('Method not implemented.');
	}
}