/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { AfterViewInit, Component, ElementRef, Input, ViewChild } from '@angular/core';
import { FormControlEditorAbstractComponent } from '../formcontrol-editor.abstract.component';

@Component({
	selector: 'goofy-client-text-editor',
	templateUrl: './text-editor.component.html',
	styleUrls: ['./text-editor.component.scss']
})
export class TextEditorComponent extends FormControlEditorAbstractComponent implements AfterViewInit {

	@ViewChild('inputElement') inputElement: ElementRef;

	@Input() label: string;
	@Input() placeholder: string;
	@Input() placeholderLabel: string = null;
	@Input() autocomplete: 'off' | 'email' = 'off';
	@Input() appearance: 'outline' | 'fill' = 'fill';
	@Input() readOnly: boolean;
	@Input() autoFocus: boolean;

	ngAfterViewInit(): void {
		window.setTimeout(() => {
			if (this.autoFocus) this.inputElement.nativeElement.focus();
		})
	}

	getPlaceholderLabel(): string {
		return this.placeholderLabel ? this.placeholderLabel : this.label;
	}
}
