/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatButton } from '@angular/material/button';
import { MatRipple } from '@angular/material/core';
import { createCommandResource } from 'libs/command-shared/test/command';
import { getDataTestClassOf } from 'libs/tech-shared/test/data-test';
import { MatTooltipDirective } from 'libs/ui/src/lib/ui/mattooltip/mattooltip.directive';
import { MockComponent } from 'ng-mocks';
import { OzgcloudButtonContentComponent } from '../shared/ozgcloud-button-content/ozgcloud-button-content.component';
import { OzgcloudButtonWithSpinnerComponent } from './ozgcloud-button-with-spinner.component';

import * as ResourceUtils from 'libs/tech-shared/src/lib/resource/resource.util';

describe('OzgcloudButtonWithSpinnerComponent', () => {
	let component: OzgcloudButtonWithSpinnerComponent;
	let fixture: ComponentFixture<OzgcloudButtonWithSpinnerComponent>;

	const buttonSelector: string = getDataTestClassOf('icon-button');

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				MatButton,
				MatRipple,
				OzgcloudButtonWithSpinnerComponent,
				MatTooltipDirective,
				MockComponent(OzgcloudButtonContentComponent)
			]
		}).compileComponents();
	});

	beforeEach(() => {
		fixture = TestBed.createComponent(OzgcloudButtonWithSpinnerComponent);
		component = fixture.componentInstance;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('isDisabled', () => {
		it('should return true if showSpinner is true and stateResource is loading', () => {
			component.showSpinner = true;
			component.stateResource.loading = true;

			const result: boolean = component.isDisabled;

			expect(result).toBeTruthy();
		})

		it('should return true if showSpinner is false and stateResource is loading', () => {
			component.showSpinner = false;
			component.stateResource.loading = true;

			const result: boolean = component.isDisabled;

			expect(result).toBeTruthy();
		})

		it('should return true if showSpinner is true and stateResource is not loading', () => {
			component.showSpinner = true;
			component.stateResource.loading = false;

			const result: boolean = component.isDisabled;

			expect(result).toBeTruthy();
		})

		it('should return false if showSpinner is false and stateResource is not loading', () => {
			component.showSpinner = false;
			component.stateResource.loading = false;

			const result: boolean = component.isDisabled;

			expect(result).toBeFalsy();
		})
	})

	describe('ngOnInit', () => {
		it('should call getStateResource()', () => {
			component.getStateResource = jest.fn();

			component.ngOnInit();

			expect(component.getStateResource).toHaveBeenCalled();
		})
	})

	describe('getStateResource', () => {
		it('should call createEmptyStateResource if stateResource is undefined', () => {
			const spy = jest.spyOn(ResourceUtils, 'createEmptyStateResource');
			component.stateResource = undefined;

			component.getStateResource();

			expect(spy).toHaveBeenCalled();
		})

		it('should call createEmptyStateResource if stateResource is null', () => {
			const spy = jest.spyOn(ResourceUtils, 'createEmptyStateResource');
			component.stateResource = null;

			component.getStateResource();

			expect(spy).toHaveBeenCalled();
		})

		it('should return non empty stateResource', () => {
			component.stateResource = ResourceUtils.createStateResource(createCommandResource());

			component.getStateResource();

			const valid: boolean = ResourceUtils.isValidStateResouce(component.stateResource);
			expect(valid).toBeTruthy();
		})
	})

	describe('Button', () => {

		it('should NOT be disabled initialy', () => {
			component.stateResource = ResourceUtils.createEmptyStateResource();
			fixture.detectChanges();

			const button = fixture.nativeElement.querySelector(buttonSelector);

			expect(button).not.toBeDisabled()
		});

		it('should be disabled if loading', () => {
			component.stateResource = ResourceUtils.createEmptyStateResource(true);
			fixture.detectChanges();

			const button = fixture.nativeElement.querySelector(buttonSelector);

			expect(button).toBeDisabled();
		});
	})
});