/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { StateResource, createEmptyStateResource } from '@goofy-client/tech-shared';
import { Resource } from '@ngxp/rest';
import { isNil } from 'lodash-es';

@Component({
	selector: 'goofy-client-ozgcloud-stroked-button-with-spinner',
	templateUrl: './ozgcloud-stroked-button-with-spinner.component.html',
	styleUrls: ['./ozgcloud-stroked-button-with-spinner.component.scss']
})
export class OzgcloudStrokedButtonWithSpinnerComponent implements OnInit {

	@Input() icon: string;
	@Input() svgIcon: string;
	@Input() text: string;
	@Input() stateResource: StateResource<Resource>;
	@Input() color: string = 'primary';
	@Input() type: 'button' | 'submit' = 'button';
	@Input() toolTip: string = '';
	@Input() dataTestId: string;
	@Input() showSpinner: boolean = false;
	@Input() strokedButton: boolean = true;

	@Output() public clickEmitter: EventEmitter<MouseEvent> = new EventEmitter<MouseEvent>();

	get isDisabled(): boolean {
		return this.showSpinner || this.stateResource.loading;
	}

	ngOnInit(): void {
		this.stateResource = this.getStateResource();
	}

	getStateResource(): StateResource<Resource> {
		return isNil(this.stateResource) ? createEmptyStateResource<Resource>() : this.stateResource;
	}
}