/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { PostfachMailLinkRel, PostfachMailResource } from '@goofy-client/postfach-shared';
import { mock } from '@goofy-client/test-utils';
import { UserProfileService } from '@goofy-client/user-profile-shared';
import { createPostfachMailResource } from 'libs/postfach-shared/test/postfach';
import { MockComponent } from 'ng-mocks';
import { UserIconComponent } from '../user-icon/user-icon.component';
import { UserProfileComponent } from '../user-profile/user-profile.component';
import { UserProfileInPostfachMailContainerComponent } from './user-profile-in-postfach-mail-container.component';

describe('UserProfileInPostfachMailComponent', () => {
	let component: UserProfileInPostfachMailContainerComponent;
	let fixture: ComponentFixture<UserProfileInPostfachMailContainerComponent>;

	const userProfileService = mock(UserProfileService);

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				UserProfileInPostfachMailContainerComponent,
				MockComponent(UserIconComponent),
				MockComponent(UserProfileComponent)
			],
			providers: [
				{
					provide: UserProfileService,
					useValue: userProfileService
				}
			]
		}).compileComponents();
	});

	beforeEach(() => {
		fixture = TestBed.createComponent(UserProfileInPostfachMailContainerComponent);
		component = fixture.componentInstance;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('ngOnChanges', () => {

		const postfachMail: PostfachMailResource = createPostfachMailResource();

		it('should call service', () => {
			component.postfachMail = postfachMail;

			component.ngOnChanges();

			expect(userProfileService.getAssignedUserProfile).toHaveBeenCalledWith(postfachMail, PostfachMailLinkRel.CREATED_BY);
		})
	})
});
