/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatIcon } from '@angular/material/icon';
import { MatIconTestingModule } from '@angular/material/icon/testing';
import { EnumToLabelPipe, HasLinkPipe } from '@goofy-client/tech-shared';
import { VorgangHeaderLinkRel } from '@goofy-client/vorgang-shared';
import { AktenzeichenComponent, VorgangNummerComponent, VorgangStatusDotComponent, VorgangStatusTextComponent } from '@goofy-client/vorgang-shared-ui';
import { UserProfileInVorgangContainerComponent } from 'libs/user-profile/src/lib/user-profile-in-vorgang-container/user-profile-in-vorgang-container.component';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { VorgangDetailHeaderComponent } from './vorgang-detail-header.component';

describe('VorgangDetailHeaderComponent', () => {
	let component: VorgangDetailHeaderComponent;
	let fixture: ComponentFixture<VorgangDetailHeaderComponent>;

	const user: string = '[data-test-id="vorgang-header-user-icon"]';

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				VorgangDetailHeaderComponent,
				EnumToLabelPipe,
				HasLinkPipe,
				MatIcon,
				MockComponent(UserProfileInVorgangContainerComponent),
				MockComponent(VorgangStatusDotComponent),
				MockComponent(VorgangStatusTextComponent),
				MockComponent(AktenzeichenComponent),
				MockComponent(VorgangNummerComponent),
			],
			imports: [
				MatIconTestingModule,
			],
		}).compileComponents();
	});

	beforeEach(() => {
		fixture = TestBed.createComponent(VorgangDetailHeaderComponent);
		component = fixture.componentInstance;
		component.vorgangWithEingang = createVorgangWithEingangResource();
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('bearbeiter/user icon', () => {

		it('should be visible', () => {
			component.vorgangWithEingang = createVorgangWithEingangResource([VorgangHeaderLinkRel.ASSIGN]);
			fixture.detectChanges();

			const userElement = fixture.nativeElement.querySelector(user);

			expect(userElement).toBeInstanceOf(HTMLElement);
		})

		it('should NOT be visible', () => {
			component.vorgangWithEingang = createVorgangWithEingangResource();
			fixture.detectChanges();

			const userElement = fixture.nativeElement.querySelector(user);

			expect(userElement).not.toBeInstanceOf(HTMLElement);
		})
	})
});
