import { Component, EventEmitter, Input, Output } from '@angular/core';
import { CommandResource } from '@goofy-client/command-shared';
import { StateResource, createEmptyStateResource, isLoaded } from '@goofy-client/tech-shared';
import { VorgangService, VorgangWithEingangResource } from '@goofy-client/vorgang-shared';
import { Observable, of, tap } from 'rxjs';

@Component({
	selector: 'goofy-client-vorgang-process-vorgang-container',
	templateUrl: './vorgang-process-vorgang-container.component.html',
	styleUrls: ['./vorgang-process-vorgang-container.component.scss'],
})
export class ProcessVorgangContainerComponent {

	@Input() vorgangWithEingang: VorgangWithEingangResource;

	@Output() public closeMenu: EventEmitter<void> = new EventEmitter();

	public commandStateResource$: Observable<StateResource<CommandResource>> = of(createEmptyStateResource<CommandResource>());

	constructor(private vorgangService: VorgangService) { }

	public processVorgang(): void {
		this.commandStateResource$ = this.vorgangService.processVorgang(this.vorgangWithEingang).pipe(
			tap(vorgangExportStateResource => this.handleCommand(vorgangExportStateResource)),
		);
	}

	handleCommand(vorgangExportStateResource: StateResource<CommandResource>): void {
		if(isLoaded(vorgangExportStateResource)) this.closeMenu.emit();
	}
}