/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatIcon } from '@angular/material/icon';
import { LoeschAnforderungService } from '@goofy-client/loesch-anforderung-shared';
import { StateResource, createStateResource } from '@goofy-client/tech-shared';
import { getDebugElementFromFixtureByCss, mock } from '@goofy-client/test-utils';
import { SpinnerComponent, SpinnerTransparencyComponent } from '@goofy-client/ui';
import { VorgangCommandService, VorgangService, VorgangWithEingang, VorgangWithEingangResource } from '@goofy-client/vorgang-shared';
import { ProgressBarComponent } from 'libs/ui/src/lib/ui/progress-bar/progress-bar.component';
import { SubnavigationComponent } from 'libs/ui/src/lib/ui/subnavigation/subnavigation.component';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { Subject } from 'rxjs';
import { VorgangDetailActionButtonsComponent } from './vorgang-detail-area/vorgang-detail-action-buttons/vorgang-detail-action-buttons.component';
import { VorgangDetailAreaComponent } from './vorgang-detail-area/vorgang-detail-area.component';
import { VorgangDetailFormularButtonsComponent } from './vorgang-detail-area/vorgang-detail-formular-buttons/vorgang-detail-formular-buttons.component';
import { VorgangDetailBackButtonContainerComponent } from './vorgang-detail-back-button-container/vorgang-detail-back-button-container.component';
import { VorgangDetailPageComponent } from './vorgang-detail-page.component';

describe('VorgangDetailPageComponent', () => {
	let component: VorgangDetailPageComponent;
	let fixture: ComponentFixture<VorgangDetailPageComponent>;

	const getVorgangWithEingangSubj: Subject<StateResource<VorgangWithEingang>> = new Subject();
	const vorgangService = { ...mock(VorgangService), getVorgangWithEingang: () => getVorgangWithEingangSubj };
	const vorgangCommandService = mock(VorgangCommandService);
	const loeschAnforderungService = mock(LoeschAnforderungService);

	const vorgangWithEingang: VorgangWithEingangResource = createVorgangWithEingangResource();

	const subnavigation: string = 'goofy-client-subnavigation';

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				VorgangDetailPageComponent,
				MatIcon,
				MockComponent(ProgressBarComponent),
				MockComponent(SubnavigationComponent),
				MockComponent(VorgangDetailAreaComponent),
				MockComponent(VorgangDetailActionButtonsComponent),
				MockComponent(VorgangDetailFormularButtonsComponent),
				MockComponent(VorgangDetailBackButtonContainerComponent),
				MockComponent(SpinnerComponent),
				MockComponent(SpinnerTransparencyComponent)
			],
			providers: [
				{
					provide: VorgangService,
					useValue: vorgangService
				},
				{
					provide: VorgangCommandService,
					useValue: vorgangCommandService
				},
				{
					provide: LoeschAnforderungService,
					useValue: loeschAnforderungService
				}
			]
		})
	});

	beforeEach(() => {
		fixture = TestBed.createComponent(VorgangDetailPageComponent);
		component = fixture.componentInstance;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('ngOnInit', () => {

		it('should get vorgangWithEingang', () => {
			vorgangService.getVorgangWithEingang = jest.fn();

			component.ngOnInit();

			expect(vorgangService.getVorgangWithEingang).toHaveBeenCalled();
		})

		it('should get getRevokeCommand', () => {
			component.ngOnInit();

			expect(vorgangCommandService.getRevokeCommand).toHaveBeenCalled();
		})

		it('should get vorgangLoeschenCommand', () => {
			component.ngOnInit();

			expect(loeschAnforderungService.getEndgueltigLoeschenCommand).toHaveBeenCalled();
		})
	})

	describe('subnavigation', () => {

		it('should be hidden', () => {
			getVorgangWithEingangSubj.next(null);
			fixture.detectChanges();

			const subnavigationElement = getDebugElementFromFixtureByCss(fixture, subnavigation);

			expect(subnavigationElement).toBeNull();
		})

		it('should be visible', () => {
			getVorgangWithEingangSubj.next(createStateResource(vorgangWithEingang));
			fixture.detectChanges();

			const subnavigationElement = getDebugElementFromFixtureByCss(fixture, subnavigation);

			expect(subnavigationElement).toBeDefined();
		})
	})
});
