/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ApiRootFacade, ApiRootLinkRel } from '@goofy-client/api-root-shared';
import { createStateResource, HasLinkPipe } from '@goofy-client/tech-shared';
import { dispatchEventFromFixture, getElementFromFixture, mock } from '@goofy-client/test-utils';
import { VorgangListService } from '@goofy-client/vorgang-shared';
import { createApiRootResource } from 'libs/api-root-shared/test/api-root';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { VorgangSearchContainerComponent } from './vorgang-search-container.component';
import { VorgangSearchComponent } from './vorgang-search/vorgang-search.component';

describe('VorgangSearchContainerComponent', () => {
	let component: VorgangSearchContainerComponent;
	let fixture: ComponentFixture<VorgangSearchContainerComponent>;

	const apiRootFacade = mock(ApiRootFacade);
	const vorgangListService = mock(VorgangListService);

	const vorgangSearch: string = getDataTestIdOf('vorgang-search');

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				VorgangSearchContainerComponent,
				HasLinkPipe,
				MockComponent(VorgangSearchComponent)
			],
			providers: [
				{
					provide: ApiRootFacade,
					useValue: apiRootFacade
				},
				{
					provide: VorgangListService,
					useValue: vorgangListService
				}
			]
		}).compileComponents();
	});

	beforeEach(() => {
		fixture = TestBed.createComponent(VorgangSearchContainerComponent);
		component = fixture.componentInstance;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('ngOnInit', () => {

		it('should call apiRootFacade to get apiRoot', () => {
			component.ngOnInit();

			expect(apiRootFacade.getApiRoot).toHaveBeenCalled();
		})

		it('should call vorgangListService to get searchPreviewList', () => {
			component.ngOnInit();

			expect(vorgangListService.getSearchPreviewList).toHaveBeenCalled();
		})

		it('should call vorgangListService to get searchString', () => {
			component.ngOnInit();

			expect(vorgangListService.getSearchString).toHaveBeenCalled();
		})
	})

	describe('vorgang-search', () => {

		it('should hide on no link exists', () => {
			component.apiRoot$ = of(createStateResource(createApiRootResource()));
			fixture.detectChanges();

			const element = getElementFromFixture(fixture, vorgangSearch);

			expect(element).not.toBeInstanceOf(HTMLElement);
		})

		it.each([ApiRootLinkRel.SEARCH_ALLE, ApiRootLinkRel.SEARCH_MEINE])
			('should show on link "%s"', (linkRel: string) => {
				component.apiRoot$ = of(createStateResource(createApiRootResource([linkRel])));
				fixture.detectChanges();

				const element = getElementFromFixture(fixture, vorgangSearch);

				expect(element).toBeInstanceOf(HTMLElement);
			});
	})

	describe('clear vorgangSearchPreviewList event', () => {

		it('should call service clearSearchPreviewList', () => {
			component.apiRoot$ = of(createStateResource(createApiRootResource([ApiRootLinkRel.SEARCH_ALLE])));
			fixture.detectChanges();

			dispatchEventFromFixture(fixture, vorgangSearch, 'clearVorgangSearchPreviewList');

			expect(vorgangListService.clearSearchPreviewList).toHaveBeenCalled();
		})
	})
});