/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BinaryFileListResource } from '@goofy-client/binary-file-shared';
import { CommandOrder, CommandResource } from '@goofy-client/command-shared';
import { StateResource, createStateResource } from '@goofy-client/tech-shared';
import { createBinaryFileListResource } from 'libs/binary-file-shared/test/binary-file';
import { createCommandResource } from 'libs/command-shared/test/command';
import { createVorgangListResource, createVorgangResources, createVorgangStatistic, createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { StatusCommandMap, VorgangFilter, VorgangListResource, VorgangResource, VorgangStatistic, VorgangView, VorgangWithEingangResource } from '../vorgang.model';
import { VorgangPartialState, initialState } from './vorgang.reducer';

import * as VorgangNavigationUtil from '../vorgang-navigation.util';
import * as VorgangSelectors from './vorgang.selectors';

describe('Vorgang Selectors', () => {

	let state: VorgangPartialState;

	const vorgangList: StateResource<VorgangListResource> = createStateResource(createVorgangListResource());
	const vorgaenge: VorgangResource[] = createVorgangResources();
	const vorgangStatistic: StateResource<VorgangStatistic> = createStateResource(createVorgangStatistic());
	const searchPreviewList: StateResource<VorgangListResource> = createStateResource(createVorgangListResource());
	const searchString: string = 'searchThisForMe';
	const vorgangFilter: VorgangFilter = VorgangFilter.ALLE;
	const vorgangView: VorgangView = VorgangView.VORGANG_LIST;

	const vorgangWithEingang: StateResource<VorgangWithEingangResource> = createStateResource(createVorgangWithEingangResource());
	const attachmentList: StateResource<BinaryFileListResource> = createStateResource(createBinaryFileListResource());
	const representationList: StateResource<BinaryFileListResource> = createStateResource(createBinaryFileListResource());
	const assignUserCommand: StateResource<CommandResource> = createStateResource(createCommandResource());
	const forwardPendingCommand: StateResource<CommandResource> = createStateResource(createCommandResource());
	const sendPostfachNachrichtPendingCommand: StateResource<CommandResource> = createStateResource(createCommandResource());

	const annehmenCommand: StateResource<CommandResource> = createStateResource(createCommandResource());
	const statusCommandMap: StatusCommandMap = <any>{ [CommandOrder.VORGANG_ANNEHMEN]: annehmenCommand };
	const revokeCommand: StateResource<CommandResource> = createStateResource(createCommandResource());
	const vorgangExport: StateResource<boolean> = createStateResource(false);

	beforeEach(() => {
		state = {
			VorgangState: {
				...initialState,
				vorgangList,
				vorgaenge,
				vorgangStatistic,
				searchPreviewList,
				searchString,
				vorgangFilter,
				vorgangView,

				vorgangWithEingang,
				attachmentList,
				representationList,
				assignUserCommand,
				forwardPendingCommand,
				sendPostfachNachrichtPendingCommand,
				statusCommandMap,
				revokeCommand,
				vorgangExport
			}
		};
	});

	it('should return vorgangList', () => {
		const result: StateResource<VorgangListResource> = VorgangSelectors.vorgangList.projector(state.VorgangState);

		expect(result).toBe(vorgangList);
	})

	it('should return vorgaenge', () => {
		const result: VorgangResource[] = VorgangSelectors.vorgaenge.projector(state.VorgangState);

		expect(result).toBe(vorgaenge);
	})

	it('should return vorgangStatistic', () => {
		const result: StateResource<VorgangStatistic> = VorgangSelectors.vorgangStatistic.projector(state.VorgangState);

		expect(result).toBe(vorgangStatistic);
	})

	it('should return vorgangFilter', () => {
		const result: VorgangFilter = VorgangSelectors.vorgangFilter.projector(state.VorgangState);

		expect(result).toBe(vorgangFilter);
	})

	it('should return vorgangView', () => {
		const result: VorgangView = VorgangSelectors.vorgangView.projector(state.VorgangState);

		expect(result).toBe(vorgangView);
	})

	it('should return searchPreviewList', () => {
		const result: StateResource<VorgangListResource> = VorgangSelectors.searchPreviewList.projector(state.VorgangState);

		expect(result).toBe(searchPreviewList);
	})

	it('should return searchString', () => {
		const result: string = VorgangSelectors.searchString.projector(state.VorgangState);

		expect(result).toBe(searchString);
	})

	describe('backButtonUrl', () => {

		it('should call vorgangNavigationUtil buildVorgangListRoutePath', () => {
			const utilSpy = jest.spyOn(VorgangNavigationUtil, 'buildBackButtonUrl');

			VorgangSelectors.backButtonUrl.projector(state.VorgangState);

			expect(utilSpy).toHaveBeenCalledWith(state.VorgangState);
		})
	})

	describe('isVorgangViewSelected', () => {

		it('should return true if state and view matches', () => {
			const result: boolean = VorgangSelectors.isVorgangViewSelected(VorgangView.VORGANG_LIST).projector(vorgangView);

			expect(result).toBeTruthy();
		})

		it('should return false if state and view does not match', () => {
			const result: boolean = VorgangSelectors.isVorgangViewSelected(VorgangView.NEU).projector(vorgangView);

			expect(result).toBeFalsy();
		})
	})

	describe('getVorgangViewRoutePath', () => {

		it('should return /alle/neu', () => {
			const result: string = VorgangSelectors.getVorgangViewRoutePath(VorgangView.NEU).projector(vorgangFilter);

			expect(result).toBe('/alle/neu');
		})
	})

	//VorgangWithEingang
	it('should return vorgangWithEingang', () => {
		const result: StateResource<VorgangWithEingangResource> = VorgangSelectors.vorgangWithEingang.projector(state.VorgangState);

		expect(result).toBe(vorgangWithEingang);
	})

	describe('attachmentList', () => {

		it('should Return attachmentList from state', () => {
			const result: StateResource<BinaryFileListResource> = VorgangSelectors.attachmentList.projector(state.VorgangState);

			expect(result).toBe(attachmentList);
		})
	})

	describe('representationList', () => {

		it('should Return representationList from state', () => {
			const result: StateResource<BinaryFileListResource> = VorgangSelectors.representationList.projector(state.VorgangState);

			expect(result).toBe(representationList);
		})
	})

	describe('forwardPendingCommand', () => {

		it('should return command from state', () => {
			const result: StateResource<CommandResource> = VorgangSelectors.forwardPendingCommand.projector(state.VorgangState);

			expect(result).toBe(forwardPendingCommand);
		})
	})

	describe('send postfach command', () => {

		it('should return command from state', () => {
			const result: StateResource<CommandResource> = VorgangSelectors.sendPostfachNachrichtPendingCommand.projector(state.VorgangState);

			expect(result).toBe(sendPostfachNachrichtPendingCommand);
		})
	})

	describe('user assign command', () => {

		it('should return command from state', () => {
			const result: StateResource<CommandResource> = VorgangSelectors.assignUserCommand.projector(state.VorgangState);

			expect(result).toBe(assignUserCommand);
		})
	})

	describe('stausCommandMap', () => {

		it('should select statusCommandMap from state', () => {
			const result: StatusCommandMap = VorgangSelectors.statusCommandMap.projector(state.VorgangState);

			expect(result).toBe(statusCommandMap);
		})
	})

	describe('getStatusCommand', () => {

		it('should return statusCommand from state by order', () => {
			const result: boolean = VorgangSelectors.getStatusCommand(CommandOrder.VORGANG_ANNEHMEN).projector(statusCommandMap);

			expect(result).toBe(annehmenCommand);
		})
	})

	describe('revokeCommand', () => {

		it('should return revokeCommand from state', () => {
			const result: StateResource<CommandResource> = VorgangSelectors.revokeCommand.projector(state.VorgangState);

			expect(result).toBe(revokeCommand);
		})
	})

	describe('exportCommand', () => {

		it('should return exportCommand from state', () => {
			const result: StateResource<boolean> = VorgangSelectors.vorgangExport.projector(state.VorgangState);

			expect(result).toBe(vorgangExport);
		})
	})});