import { Injectable, OnDestroy } from "@angular/core";
import { FormControl, FormGroup, UntypedFormBuilder } from "@angular/forms";
import { NavigationService } from "@goofy-client/navigation-shared";
import { isNotNil } from "@goofy-client/tech-shared";
import { VorgangListService } from "@goofy-client/vorgang-shared";
import { Subscription } from "rxjs";

@Injectable()
export class VorgangFilterFormService implements OnDestroy {

	public form: FormGroup;

	static readonly FILTER: string = 'filter';

	public readonly FILTER_ALLE_VALUE: string = 'alle';
	public readonly FILTER_MEINE_VALUE: string = 'meine';
	public readonly FILTER_UNASSIGNED_VALUE: string = 'unassigned';

	private subscription: Subscription;

	constructor(
		private formBuilder: UntypedFormBuilder,
		private navigationService: NavigationService,
		private vorgangListService: VorgangListService
	) {
		this.init();
	}

	init(): void {
		this.initForm();

		this.subscribeToFilterChanges();
		this.subscribeToValueChanges();
	}

	private initForm(): void {
		this.form = this.formBuilder.group({
			[VorgangFilterFormService.FILTER]: new FormControl()
		})
	}

	subscribeToFilterChanges(): void {
		this.subscription = this.vorgangListService.getSelectedFilterAsRouteParam().subscribe((filter: string) => this.patch(filter));
	}

	private patch(filter: string): void {
		this.getFilterControl().patchValue(filter);
	}

	subscribeToValueChanges(): void {
		this.subscription = this.getFilterControl().valueChanges.subscribe(value => this.handleFilterChange(value));
	}

	private getFilterControl(): FormControl {
		return <FormControl>this.form.controls[VorgangFilterFormService.FILTER];
	}

	private handleFilterChange(value: string): void {
		this.navigationService.navigateByFilter(value);
	}

	ngOnDestroy(): void {
		if (isNotNil(this.subscription)) this.subscription.unsubscribe();
	}
}
