/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { SimpleChanges } from '@angular/core';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { UntypedFormBuilder, ReactiveFormsModule } from '@angular/forms';
import { BinaryFileAttachmentContainerComponent } from '@goofy-client/binary-file';
import { BinaryFileListResource } from '@goofy-client/binary-file-shared';
import { Mock, mock, useFromMock } from '@goofy-client/test-utils';
import { DateEditorComponent, TextAreaEditorComponent, TextEditorComponent } from '@goofy-client/ui';
import { WiedervorlageLinkRel, WiedervorlageResource, WiedervorlageService } from '@goofy-client/wiedervorlage-shared';
import { createBinaryFileListResource } from 'libs/binary-file-shared/test/binary-file';
import { createWiedervorlageResource } from 'libs/wiedervorlage-shared/test/wiedervorlage';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { SubmitWiedervorlageButtonComponent } from '../../../submit-wiedervorlage-button/submit-wiedervorlage-button.component';
import { WiedervorlageFormComponent } from './wiedervorlage-form.component';
import { WiedervorlageFormService } from './wiedervorlage.formservice';

describe('WiedervorlageFormComponent', () => {
	let component: WiedervorlageFormComponent;
	let fixture: ComponentFixture<WiedervorlageFormComponent>;

	const formService = new WiedervorlageFormService(new UntypedFormBuilder(), useFromMock(mock(WiedervorlageService)));
	const wiedervorlageService: Mock<WiedervorlageService> = mock(WiedervorlageService);
	const wiedervorlage: WiedervorlageResource = createWiedervorlageResource();

	beforeEach(async () => {
		await TestBed.configureTestingModule({
			declarations: [
				WiedervorlageFormComponent,
				MockComponent(DateEditorComponent),
				MockComponent(TextEditorComponent),
				MockComponent(TextAreaEditorComponent),
				MockComponent(BinaryFileAttachmentContainerComponent),
				MockComponent(SubmitWiedervorlageButtonComponent)
			],
			imports: [
				ReactiveFormsModule
			],
			providers: [
				{
					provide: WiedervorlageFormService,
					useValue: formService
				},
				{
					provide: WiedervorlageService,
					useValue: wiedervorlageService
				}
			]
		})
	});

	beforeEach(() => {
		formService.patch = jest.fn();

		fixture = TestBed.createComponent(WiedervorlageFormComponent);
		component = fixture.componentInstance;
		fixture.detectChanges();
	});

	it('should create', () => {
		expect(component).toBeTruthy();
	});

	describe('ngOnChanges', () => {

		const simpleChanges: SimpleChanges = <any>{ wiedervorlage };

		beforeEach(() => {
			component.wiedervorlage = wiedervorlage;
			component.patchWiedervorlage = jest.fn();
			component.updateAttachments = jest.fn();
		})

		it('should call patchWiedervorlage', () => {
			component.ngOnChanges(simpleChanges);

			expect(component.patchWiedervorlage).toHaveBeenCalled();
		})

		it('should call updateAttachments', () => {
			component.ngOnChanges(simpleChanges);

			expect(component.updateAttachments).toHaveBeenCalled();
		})
	})

	describe('updateAttachments', () => {

		const binaryFileListResource: BinaryFileListResource = createBinaryFileListResource();

		beforeEach(() => {
			wiedervorlageService.getAttachmentList.mockReturnValue(of(binaryFileListResource));
		})

		it('should call wiedervorlage service to get attachments if links exists', () => {
			const wiedervorlageWithAttachments: WiedervorlageResource = createWiedervorlageResource([WiedervorlageLinkRel.ATTACHMENTS]);
			component.wiedervorlage = wiedervorlageWithAttachments;

			component.updateAttachments();

			expect(wiedervorlageService.getAttachmentList).toHaveBeenCalledWith(wiedervorlageWithAttachments);
		})

		it('should NOT call wiedervorlage service if link is not present', () => {
			component.wiedervorlage = wiedervorlage;

			component.updateAttachments();

			expect(wiedervorlageService.getAttachmentList).not.toHaveBeenCalledWith(wiedervorlage);
		})
	})

	describe('patch wiedervorlage', () => {

		it('should NOT patch by formservice', () => {
			component.wiedervorlage = null;

			component.patchWiedervorlage();

			expect(formService.patch).not.toHaveBeenCalled();
		})

		it('should patch by formservice', () => {
			component.wiedervorlage = wiedervorlage;

			component.patchWiedervorlage();

			expect(formService.patch).toHaveBeenCalledWith(wiedervorlage);
		})
	})
});
