{{/* vim: set filetype=mustache: */}}

{{/* Truncate at 63 chars because some Kubernetes name fields are limited to this (by the DNS naming spec) */}}
{{/* Name */}}
{{- define "app.name" -}}
{{- default .Release.Name | toString | trunc 63 | trimSuffix "-" -}}
{{- end -}}

{{/* Namespace */}}
{{- define "app.namespace" -}}
{{- default .Release.Namespace | toString | trunc 63 | trimSuffix "-" -}}
{{- end -}}

{{/* Version */}}
{{- define "app.version" -}}
{{- default .Chart.Version | toString | trunc 63 | trimSuffix "-" -}}
{{- end -}}

{{/* Chart: Name + Version */}}
{{- define "app.chart" -}}
{{ printf "%s-%s" .Chart.Name .Chart.Version | replace "+" "_" | trunc 63 | trimSuffix "-" }}
{{- end -}}

{{/* Managed-by -> On Helm, this value is always Helm */}}
{{- define "app.managedBy" -}}
{{- default .Release.Service | toString | trunc 63 | trimSuffix "-" -}}
{{- end -}}

{{/* Default Labels: Helm recommended best-practice labels https://helm.sh/docs/chart_best_practices/labels/ */}}
{{- define "app.defaultLabels" }}
app.kubernetes.io/instance: goofy
app.kubernetes.io/managed-by: {{ include "app.managedBy" . }}
app.kubernetes.io/name: {{ include "app.name" . }}
app.kubernetes.io/part-of: kop
app.kubernetes.io/version: {{ include "app.version" . }}
app.kubernetes.io/namespace: {{ include "app.namespace" . }}
helm.sh/chart: {{ include "app.chart" . }}
{{- end -}}

{{- define "app.matchLabels" }}
app.kubernetes.io/name: {{ include "app.name" . }}
app.kubernetes.io/namespace: {{ include "app.namespace" . }}
{{- end -}}

{{- define "app.envSpringProfiles" }}
{{- if (.Values.env).overrideSpringProfiles -}}
{{ printf "%s" (.Values.env).overrideSpringProfiles }}
{{- else -}}
{{ printf "oc, %s" (include "app.kopEnvironment" . ) }}
{{- end -}}
{{- end -}}

{{- define "app.grpc_client_pluto_address" -}}
{{ printf "%s.%s:9090" ( coalesce .Values.plutoName "pluto" ) .Release.Namespace | replace "+" "_" | trunc 63 | trimSuffix "-" }}
{{- end -}}

{{- define "app.grpc_client_user-manager_address" -}}
{{ printf "%s.%s:9000" ( coalesce .Values.usermanagerName "user-manager" ) .Release.Namespace }}
{{- end -}}

{{- define "app.kop_user-manager_url" -}}
{{- if (.Values).usermanagerUrl -}}
{{ printf "%s" (.Values).usermanagerUrl }}
{{- else -}}
{{ printf "https://%s-%s.%s" (include "app.kopBezeichner" .) .Values.usermanagerName .Values.baseUrl }}
{{- end -}}
{{- end -}}

{{- define "app.imagePullSecret" }}
{{- with .Values.imageCredentials }}
{{- printf "{\"auths\":{\"%s\":{\"username\":\"%s\",\"password\":\"%s\",\"email\":\"%s\",\"auth\":\"%s\"}}}" .registry .username .password .email (printf "%s:%s" .username .password | b64enc) | b64enc }}
{{- end }}
{{- end }}

{{- define "app.kopBundesland" -}}
{{- required "Bundesland muss angegeben sein" (.Values.kop).bundesland }}
{{- end -}}

{{- define "app.kopBezeichner" -}}
{{ $length := len (.Values.kop).bezeichner }}
{{- if ge 27 $length -}}
{{- required "Bezeichner muss angegeben sein" (.Values.kop).bezeichner -}}
{{- else -}}
{{ required (printf "Bezeichner %s ist zu lang (max. 27 Zeichen)" (.Values.kop).bezeichner) nil }}
{{- end -}}
{{- end -}}

{{- define "app.kopEnvironment" -}}
{{- required "Environment muss angegeben sein" (.Values.kop).environment -}}
{{- end -}}

{{- define "app.ssoRealm" -}}
{{ printf "%s-%s-%s" (include "app.kopBundesland" .) ( include "app.kopBezeichner" . ) ( include "app.kopEnvironment" . ) | trunc 63 | trimSuffix "-" }}
{{- end -}}

{{- define "app.ssoClientName" -}}
{{- if (.Values.sso).client_name -}}
{{ printf "%s" (.Values.sso).client_name }}
{{- else -}}
{{ printf "alfa" }}
{{- end -}}
{{- end -}}

{{- define "app.ssoRealmDisplayName" -}}
{{- if (.Values.sso).displayName -}}
{{ printf "%s" (.Values.sso).displayName }}
{{- else if eq (include "app.kopEnvironment" . ) "prod" -}}
{{ printf "Realm für %s" (include "app.kopBezeichner" . | title ) }}
{{- else -}}
{{ printf "Realm für %s (%s)" (include "app.kopBezeichner" . | title ) (include "app.kopEnvironment" .) }}
{{- end -}}
{{- end -}}

{{- define "app.ssoServerUrl" -}}
{{- printf "%s" .Values.sso.serverUrl -}}
{{- end -}}

{{- define "app.baseUrl" -}}
{{ printf "https://%s.%s" (include "app.kopBezeichner" .) .Values.baseUrl }}
{{- end -}}

{{- define "app.generateKeycloakGroupRessourceName" -}}
{{- $kebabcaseGroupName := . | title | nospace | kebabcase }}
{{- $alphanumericGroupName :=  (include "app.replaceAllNonAlphanumeric" $kebabcaseGroupName ) }}
{{- printf "%s-keycloak-group"  ( $alphanumericGroupName | trunc 20) }}
{{- end -}}

{{- define "app.replaceAllNonAlphanumeric" -}}
{{- trimAll "-" ( regexReplaceAll "[^a-zA-Z0-9-]" . "" ) }}
{{- end -}}

{{- define "app.generateKeycloakUserRessourceName" -}}
{{- $lowerUserName := . | lower }}
{{- $alphanumericUserName :=  (include "app.replaceAllNonAlphanumeric" $lowerUserName ) }}
{{- printf "%s-keycloak-user"  ( $alphanumericUserName | trunc 20) }}
{{- end -}}

{{- define "app.generateKeycloakClientRessourceName" -}}
{{- $lowerClientName := . | lower }}
{{- $alphanumericClientName :=  (include "app.replaceAllNonAlphanumeric" $lowerClientName ) }}
{{- printf "%s-keycloak-client"  ( $alphanumericClientName | trunc 20) }}
{{- end -}}